# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: pymammotion/proto/basestation.proto, pymammotion/proto/common.proto, pymammotion/proto/dev_net.proto, pymammotion/proto/luba_msg.proto, pymammotion/proto/luba_mul.proto, pymammotion/proto/mctrl_driver.proto, pymammotion/proto/mctrl_nav.proto, pymammotion/proto/mctrl_ota.proto, pymammotion/proto/mctrl_pept.proto, pymammotion/proto/mctrl_sys.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass
from typing import (
    Optional,
)

import betterproto



def has_field(message: betterproto.Message) -> bool:
    """Check if the message has any fields serialized on wire."""
    return betterproto.serialized_on_wire(message)


class WifiConfType(betterproto.Enum):
    DisconnectWifi = 0
    ForgetWifi = 1
    DirectConnectWifi = 2
    ReconnectWifi = 3
    set_enable = 4



class DrvUploadFileFileType(betterproto.Enum):
    FILE_TYPE_ALL = 0
    FILE_TYPE_SYSLOG = 1
    FILE_TYPE_NAVLOG = 2
    FILE_TYPE_RTKLOG = 3



class DrvDevInfoResult(betterproto.Enum):
    DRV_RESULT_FAIL = 0
    DRV_RESULT_SUC = 1
    DRV_RESULT_NOTSUP = 2



class SimCardSta(betterproto.Enum):
    SIM_NONE = 0
    SIM_NO_CARD = 1
    SIM_INVALID = 2
    SIM_INPUT_PIN = 3
    SIM_INPUT_PUK = 4
    SIM_OK = 5



class MnetLinkType(betterproto.Enum):
    MNET_LINK_NONE = 0
    MNET_LINK_2G = 1
    MNET_LINK_3G = 2
    MNET_LINK_4G = 3



class ApnAuthType(betterproto.Enum):
    APN_AUTH_NONE = 0
    APN_AUTH_PAP = 1
    APN_AUTH_CHAP = 2
    APN_AUTH_PAP_CHAP = 3



class NetType(betterproto.Enum):
    WIFI = 0
    MNET = 1



class IotConctrlType(betterproto.Enum):
    IOT_TYPE_OFFLINE = 0
    IOT_TYPE_ONLINE = 1
    IOT_TYPE_RESET = 2



class Operation(betterproto.Enum):
    WRITE = 0
    READ = 1
    ERASE = 2



class OffPartId(betterproto.Enum):
    OFF_PART_DL_IMG = 0
    OFF_PART_UPDINFO_BACK = 1
    OFF_PART_UPDINFO = 2
    OFF_PART_NAKEDB = 3
    OFF_PART_FLASHDB = 4
    OFF_PART_UPD_APP_IMG = 5
    OFF_PART_UPD_BMS_IMG = 6
    OFF_PART_UPD_TMP_IMG = 7
    OFF_PART_DEV_INFO = 8
    OFF_PART_NAKEDB_BACK = 9
    OFF_PART_MAX = 10



class QcAppTestId(betterproto.Enum):
    QC_APP_ITEM_ON_CHARGESATSTION = 0
    QC_APP_TEST_X3_SPEAKER = 1
    QC_APP_TEST_STATIC_OBSTACLE_DETECTION = 2
    QC_APP_TEST_CHARGESTATION_TEMP = 3
    QC_APP_ITEM_KEY = 4
    QC_APP_TEST_BUMPER_FRONTLEFT = 5
    QC_APP_TEST_BUMPER_FRONTRIGHT = 6
    QC_APP_TEST_STOP = 7
    QC_APP_TEST_UNLOCK = 8
    QC_APP_TEST_BUZZ = 9
    QC_APP_TEST_LIFT = 10
    QC_APP_ITEM_SENEOR = 11
    QC_APP_TEST_ROLL_LEFT = 12
    QC_APP_TEST_ROLL_RIGHT = 13
    QC_APP_TEST_ULTRA_UNCOVER = 14
    QC_APP_TEST_ULTRA0_COVER = 15
    QC_APP_TEST_ULTRA1_COVER = 16
    QC_APP_TEST_ULTRA2_COVER = 17
    QC_APP_TEST_RAIN = 18
    QC_APP_ITEM_SQ = 19
    QC_APP_TEST_BLE_RSSI = 20
    QC_APP_TEST_SATELLITES_ROVER = 21
    QC_APP_TEST_SATELLITES_REF_STATION_L1 = 22
    QC_APP_TEST_SATELLITES_REF_STATION_L2 = 23
    QC_APP_TEST_SATELLITES_COMMON_VIEW = 24
    QC_APP_TEST_CNO_ROVER = 25
    QC_APP_TEST_CNO_REF_STATION = 26
    QC_APP_TEST_REF_STATION_LINK_STATUS = 27
    QC_APP_TEST_LOCATION_STATE = 28
    QC_APP_TEST_MAX = 29



class NetUsedType(betterproto.Enum):
    NONE = 0
    WIFI = 1
    MNET = 2



class RptInfoType(betterproto.Enum):
    RIT_CONNECT = 0
    RIT_DEV_STA = 1
    RIT_RTK = 2
    RIT_DEV_LOCAL = 3
    RIT_WORK = 4
    RIT_FW_INFO = 5
    RIT_MAINTAIN = 6
    RIT_VISION_POINT = 7
    RIT_VIO = 8
    RIT_VISION_STATISTIC = 9
    RIT_BASESTATION_INFO = 10



class RptAct(betterproto.Enum):
    RPT_START = 0
    RPT_STOP = 1
    RPT_KEEP = 2



class InfoType(betterproto.Enum):
    IT_BASE = 0
    IT_OTA = 1



class MulLanguage(betterproto.Enum):
    ENGLISH = 0
    GERMAN = 1
    UNRECOGNIZED = -1



class MulCameraPosition(betterproto.Enum):
    LEFT = 0
    RIGHT = 1
    REAR = 2
    ALL = 3



class MulVideoErrorCode(betterproto.Enum):
    SUCCESS = 0
    ACTIVATION_FAILED = 1
    NETWORK_NOT_AVAILABLE = 2
    CREATE_CHANNEL_FAILED = 3
    PARAM_INVAILD = 4
    CELLULAR_RESTRICTION = 5



class MulWiperErrorCode(betterproto.Enum):
    SET_SUCCESS = 0
    HW_ERROR = 1
    NAVIGATION_WORK_FORBID = 2



class MsgCmdType(betterproto.Enum):
    START = 0
    NAV = 240
    LOCALIZATION = 241
    PLANNING = 242
    EMBED_DRIVER = 243
    EMBED_SYS = 244
    EMBED_MIDWARE = 245
    EMBED_OTA = 246
    APPLICATION = 247
    ESP = 248
    MUL = 249
    PEPT = 250



class MsgAttr(betterproto.Enum):
    NONE = 0
    REQ = 1
    RESP = 2
    REPORT = 3



class MsgDevice(betterproto.Enum):
    DEV_COMM_ESP = 0
    DEV_MAINCTL = 1
    DEV_LEFTMOTOR = 2
    DEV_RIGHTMOTOR = 3
    DEV_BASESTATION = 4
    DEV_RTKCLI = 5
    DEV_USBHOST = 6
    DEV_MOBILEAPP = 7
    DEV_IOTSERVER = 8
    DEV_BMS = 9
    DEV_NAVIGATION = 17
    DEV_LOCALIZATION = 18
    DEV_PERCEPTION = 19
    SOC_MODULE_MULTIMEDIA = 21
    DEV_IOTCTRL = 25



@dataclass(eq=False, repr=False)
class RequestBasestationInfoT(betterproto.Message):
    request_type: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class ResponseBasestationInfoT(betterproto.Message):
    system_status: int = betterproto.uint64_field(1)
    app_connect_type: int = betterproto.uint32_field(2)
    ble_rssi: int = betterproto.int32_field(3)
    wifi_rssi: int = betterproto.int32_field(4)
    sats_num: int = betterproto.uint64_field(5)
    lora_scan: int = betterproto.int64_field(6)
    lora_channel: int = betterproto.int64_field(7)
    lora_locid: int = betterproto.int64_field(8)
    lora_netid: int = betterproto.int64_field(9)
    rtk_status: int = betterproto.uint64_field(10)
    lowpower_status: int = betterproto.int32_field(11)
    mqtt_rtk_status: int = betterproto.int32_field(12)
    rtk_channel: int = betterproto.int32_field(13)
    rtk_switch: int = betterproto.int32_field(14)


@dataclass(eq=False, repr=False)
class AppToBaseMqttRtkT(betterproto.Message):
    rtk_switch: int = betterproto.int32_field(1)
    rtk_url: str = betterproto.string_field(2)
    rtk_port: int = betterproto.int32_field(3)
    rtk_username: str = betterproto.string_field(4)
    rtk_password: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class BaseToAppMqttRtkT(betterproto.Message):
    rtk_switch_status: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class BaseStation(betterproto.Message):
    to_dev: "RequestBasestationInfoT" = betterproto.message_field(
        1, group="BaseStationSubType"
    )
    to_app: "ResponseBasestationInfoT" = betterproto.message_field(
        2, group="BaseStationSubType"
    )
    app_to_base_mqtt_rtk_msg: "AppToBaseMqttRtkT" = betterproto.message_field(
        3, group="BaseStationSubType"
    )
    base_to_app_mqtt_rtk_msg: "BaseToAppMqttRtkT" = betterproto.message_field(
        4, group="BaseStationSubType"
    )



@dataclass(eq=False, repr=False)
class CommDataCouple(betterproto.Message):
    x: float = betterproto.float_field(1)
    y: float = betterproto.float_field(2)


@dataclass(eq=False, repr=False)
class DrvWifiUpload(betterproto.Message):
    wifi_msg_upload: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class DrvWifiList(betterproto.Message):
    nvs_wifi_upload: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class DrvWifiSet(betterproto.Message):
    config_param: int = betterproto.int32_field(1)
    confssid: str = betterproto.string_field(2)
    wifi_enable: bool = betterproto.bool_field(3)


@dataclass(eq=False, repr=False)
class DrvWifiMsg(betterproto.Message):
    status1: bool = betterproto.bool_field(1)
    status2: bool = betterproto.bool_field(2)
    ip: str = betterproto.string_field(3)
    msgssid: str = betterproto.string_field(4)
    password: str = betterproto.string_field(5)
    rssi: int = betterproto.int32_field(6)
    productkey: str = betterproto.string_field(7)
    devicename: str = betterproto.string_field(8)
    wifi_enable: bool = betterproto.bool_field(9)


@dataclass(eq=False, repr=False)
class DrvWifiConf(betterproto.Message):
    succ_flag: bool = betterproto.bool_field(1)
    code: int = betterproto.int32_field(2)
    confssid: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class DrvListUpload(betterproto.Message):
    sum: int = betterproto.int32_field(1)
    current: int = betterproto.int32_field(2)
    status: int = betterproto.int32_field(3)
    memssid: str = betterproto.string_field(4)
    rssi: int = betterproto.int32_field(5)


@dataclass(eq=False, repr=False)
class DrvUploadFileReq(betterproto.Message):
    biz_id: str = betterproto.string_field(1)
    url: str = betterproto.string_field(2)
    user_id: str = betterproto.string_field(3)
    num: int = betterproto.int32_field(4)
    type: int = betterproto.int32_field(5)


@dataclass(eq=False, repr=False)
class DrvUploadFileCancel(betterproto.Message):
    biz_id: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class DrvUploadFileToAppReq(betterproto.Message):
    biz_id: str = betterproto.string_field(1)
    operation: int = betterproto.int32_field(2)
    server_ip: int = betterproto.fixed32_field(3)
    server_port: int = betterproto.int32_field(4)
    num: int = betterproto.int32_field(5)
    type: int = betterproto.int32_field(6)


@dataclass(eq=False, repr=False)
class DrvUploadFileToAppRsp(betterproto.Message):
    biz_id: str = betterproto.string_field(1)
    operation: int = betterproto.int32_field(2)
    result: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class DrvDevInfoReqId(betterproto.Message):
    id: int = betterproto.int32_field(1)
    type: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class DrvDevInfoRespId(betterproto.Message):
    id: int = betterproto.int32_field(1)
    type: int = betterproto.int32_field(2)
    res: "DrvDevInfoResult" = betterproto.enum_field(3)
    info: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class DrvDevInfoReq(betterproto.Message):
    req_ids: list["DrvDevInfoReqId"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class DrvDevInfoResp(betterproto.Message):
    resp_ids: list["DrvDevInfoRespId"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class DrvUpgradeReport(betterproto.Message):
    devname: str = betterproto.string_field(1)
    otaid: str = betterproto.string_field(2)
    version: str = betterproto.string_field(3)
    progress: int = betterproto.int32_field(4)
    result: int = betterproto.int32_field(5)
    message: str = betterproto.string_field(6)
    properties: str = betterproto.string_field(7)


@dataclass(eq=False, repr=False)
class WifiIotStatusReport(betterproto.Message):
    wifi_connected: bool = betterproto.bool_field(1)
    iot_connected: bool = betterproto.bool_field(2)
    productkey: str = betterproto.string_field(3)
    devicename: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class BleTestBytes(betterproto.Message):
    seqs: int = betterproto.int32_field(1)
    data: list[int] = betterproto.fixed32_field(2)


@dataclass(eq=False, repr=False)
class GetNetworkInfoReq(betterproto.Message):
    req_ids: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class GetNetworkInfoRsp(betterproto.Message):
    req_ids: int = betterproto.int32_field(1)
    wifi_ssid: str = betterproto.string_field(2)
    wifi_mac: str = betterproto.string_field(3)
    wifi_rssi: int = betterproto.int32_field(4)
    ip: int = betterproto.fixed32_field(5)
    mask: int = betterproto.fixed32_field(6)
    gateway: int = betterproto.fixed32_field(7)


@dataclass(eq=False, repr=False)
class MnetInetStatus(betterproto.Message):
    connect: bool = betterproto.bool_field(1)
    ip: int = betterproto.fixed32_field(2)
    mask: int = betterproto.fixed32_field(3)
    gateway: int = betterproto.fixed32_field(4)


@dataclass(eq=False, repr=False)
class MnetInfo(betterproto.Message):
    model: str = betterproto.string_field(1)
    revision: str = betterproto.string_field(2)
    imei: str = betterproto.string_field(3)
    sim: "SimCardSta" = betterproto.enum_field(4)
    imsi: str = betterproto.string_field(5)
    link_type: "MnetLinkType" = betterproto.enum_field(6)
    rssi: int = betterproto.int32_field(7)
    inet: "MnetInetStatus" = betterproto.message_field(8)
    iccid: str = betterproto.string_field(9)


@dataclass(eq=False, repr=False)
class GetMnetInfoReq(betterproto.Message):
    req_ids: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class GetMnetInfoRsp(betterproto.Message):
    req_ids: int = betterproto.int32_field(1)
    result: int = betterproto.int32_field(2)
    mnet: "MnetInfo" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class MnetApn(betterproto.Message):
    cid: int = betterproto.int32_field(1)
    apn_alias: str = betterproto.string_field(2)
    apn_name: str = betterproto.string_field(3)
    auth: "ApnAuthType" = betterproto.enum_field(4)
    username: str = betterproto.string_field(5)
    password: str = betterproto.string_field(6)


@dataclass(eq=False, repr=False)
class MnetApnCfg(betterproto.Message):
    apn_used_idx: int = betterproto.int32_field(1)
    apn: list["MnetApn"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class MnetApnSetCfg(betterproto.Message):
    use_default: bool = betterproto.bool_field(1)
    cfg: "MnetApnCfg" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class MnetCfg(betterproto.Message):
    mnet_enable: bool = betterproto.bool_field(1)
    inet_enable: bool = betterproto.bool_field(2)
    type: "NetType" = betterproto.enum_field(3)
    apn: "MnetApnSetCfg" = betterproto.message_field(4)
    auto_select: bool = betterproto.bool_field(5)


@dataclass(eq=False, repr=False)
class GetMnetCfgReq(betterproto.Message):
    req_ids: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class GetMnetCfgRsp(betterproto.Message):
    req_ids: int = betterproto.int32_field(1)
    result: int = betterproto.int32_field(2)
    cfg: "MnetCfg" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class SetMnetCfgReq(betterproto.Message):
    req_ids: int = betterproto.int32_field(1)
    cfg: "MnetCfg" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class SetMnetCfgRsp(betterproto.Message):
    req_ids: int = betterproto.int32_field(1)
    result: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class DrvDebugDdsZmq(betterproto.Message):
    is_enable: bool = betterproto.bool_field(1)
    rx_topic_name: str = betterproto.string_field(2)
    tx_zmq_url: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class SetDrvBleMtu(betterproto.Message):
    mtu_count: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class DevNet(betterproto.Message):
    todev_ble_sync: int = betterproto.int32_field(
        1, group="NetSubType"
    )
    todev_conf_type: "WifiConfType" = betterproto.enum_field(
        2, group="NetSubType"
    )
    todev_wifi_msg_upload: "DrvWifiUpload" = betterproto.message_field(
        3, group="NetSubType"
    )
    todev_wifi_list_upload: "DrvWifiList" = betterproto.message_field(
        4, group="NetSubType"
    )
    todev_wifi_configuration: "DrvWifiSet" = betterproto.message_field(
        5, group="NetSubType"
    )
    toapp_wifi_msg: "DrvWifiMsg" = betterproto.message_field(
        6, group="NetSubType"
    )
    toapp_wifi_conf: "DrvWifiConf" = betterproto.message_field(
        7, group="NetSubType"
    )
    toapp_list_upload: "DrvListUpload" = betterproto.message_field(
        8, group="NetSubType"
    )
    todev_req_log_info: "DrvUploadFileReq" = betterproto.message_field(
        9, group="NetSubType"
    )
    todev_log_data_cancel: "DrvUploadFileCancel" = betterproto.message_field(
        10, group="NetSubType"
    )
    todev_devinfo_req: "DrvDevInfoReq" = betterproto.message_field(
        11, group="NetSubType"
    )
    toapp_devinfo_resp: "DrvDevInfoResp" = betterproto.message_field(
        12, group="NetSubType"
    )
    toapp_upgrade_report: "DrvUpgradeReport" = betterproto.message_field(
        13, group="NetSubType"
    )
    toapp_wifi_iot_status: "WifiIotStatusReport" = betterproto.message_field(
        14, group="NetSubType"
    )
    todev_uploadfile_req: "DrvUploadFileToAppReq" = betterproto.message_field(
        15, group="NetSubType"
    )
    toapp_uploadfile_rsp: "DrvUploadFileToAppRsp" = betterproto.message_field(
        16, group="NetSubType"
    )
    todev_networkinfo_req: "GetNetworkInfoReq" = betterproto.message_field(
        17, group="NetSubType"
    )
    toapp_networkinfo_rsp: "GetNetworkInfoRsp" = betterproto.message_field(
        18, group="NetSubType"
    )
    bir_testdata: "BleTestBytes" = betterproto.message_field(
        19, group="NetSubType"
    )
    todev_mnet_info_req: "GetMnetInfoReq" = betterproto.message_field(
        20, group="NetSubType"
    )
    toapp_mnet_info_rsp: "GetMnetInfoRsp" = betterproto.message_field(
        21, group="NetSubType"
    )
    todev_get_mnet_cfg_req: "GetMnetCfgReq" = betterproto.message_field(
        22, group="NetSubType"
    )
    toapp_get_mnet_cfg_rsp: "GetMnetCfgRsp" = betterproto.message_field(
        23, group="NetSubType"
    )
    todev_set_mnet_cfg_req: "SetMnetCfgReq" = betterproto.message_field(
        24, group="NetSubType"
    )
    toapp_set_mnet_cfg_rsp: "SetMnetCfgRsp" = betterproto.message_field(
        25, group="NetSubType"
    )
    todev_set_dds2_zmq: "DrvDebugDdsZmq" = betterproto.message_field(
        26, group="NetSubType"
    )
    todev_set_ble_mtu: "SetDrvBleMtu" = betterproto.message_field(
        27, group="NetSubType"
    )
    todev_set_iot_offline_req: "IotConctrlType" = betterproto.enum_field(
        28, group="NetSubType"
    )



@dataclass(eq=False, repr=False)
class DrvMotionCtrl(betterproto.Message):
    set_linear_speed: int = betterproto.int32_field(1)
    set_angular_speed: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class DrvKnifeHeight(betterproto.Message):
    knife_height: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class DrvSrSpeed(betterproto.Message):
    rw: int = betterproto.int32_field(1)
    speed: float = betterproto.float_field(2)


@dataclass(eq=False, repr=False)
class DrvKnifeStatus(betterproto.Message):
    knife_status: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class DrvKnifeChangeReport(betterproto.Message):
    is_start: int = betterproto.int32_field(1)
    start_high: int = betterproto.int32_field(2)
    end_high: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class DrvMowCtrlByHand(betterproto.Message):
    main_ctrl: int = betterproto.int32_field(1)
    cut_knife_ctrl: int = betterproto.int32_field(2)
    cut_knife_height: int = betterproto.int32_field(3)
    max_run_speed: float = betterproto.float_field(4)


@dataclass(eq=False, repr=False)
class RtkCfgReqT(betterproto.Message):
    cmd_length: int = betterproto.int32_field(1)
    cmd_req: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class RtkCfgReqAckT(betterproto.Message):
    cmd_length: int = betterproto.int32_field(1)
    cmd_response: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class RtkSysMaskQueryT(betterproto.Message):
    sat_system: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class RtkSysMaskQueryAckT(betterproto.Message):
    sat_system: int = betterproto.uint32_field(1)
    system_mask_bits: list[int] = betterproto.uint32_field(2)


@dataclass(eq=False, repr=False)
class MctlDriver(betterproto.Message):
    todev_devmotion_ctrl: "DrvMotionCtrl" = betterproto.message_field(
        1, group="SubDrvMsg"
    )
    todev_knife_height_set: "DrvKnifeHeight" = betterproto.message_field(
        2, group="SubDrvMsg"
    )
    bidire_speed_read_set: "DrvSrSpeed" = betterproto.message_field(
        3, group="SubDrvMsg"
    )
    bidire_knife_height_report: "DrvKnifeHeight" = betterproto.message_field(
        4, group="SubDrvMsg"
    )
    toapp_knife_status: "DrvKnifeStatus" = betterproto.message_field(
        5, group="SubDrvMsg"
    )
    mow_ctrl_by_hand: "DrvMowCtrlByHand" = betterproto.message_field(
        6, group="SubDrvMsg"
    )
    rtk_cfg_req: "RtkCfgReqT" = betterproto.message_field(
        7, group="SubDrvMsg"
    )
    rtk_cfg_req_ack: "RtkCfgReqAckT" = betterproto.message_field(
        8, group="SubDrvMsg"
    )
    rtk_sys_mask_query: "RtkSysMaskQueryT" = betterproto.message_field(
        9, group="SubDrvMsg"
    )
    rtk_sys_mask_query_ack: "RtkSysMaskQueryAckT" = betterproto.message_field(
        10, group="SubDrvMsg"
    )
    toapp_knife_status_change: "DrvKnifeChangeReport" = (
        betterproto.message_field(11, group="SubDrvMsg")
    )



@dataclass(eq=False, repr=False)
class NavLatLonUp(betterproto.Message):
    lat: float = betterproto.double_field(1)
    lon: float = betterproto.double_field(2)


@dataclass(eq=False, repr=False)
class NavBorderState(betterproto.Message):
    bdstate: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class NavPosUp(betterproto.Message):
    x: float = betterproto.float_field(1)
    y: float = betterproto.float_field(2)
    status: int = betterproto.int32_field(3)
    toward: int = betterproto.int32_field(4)
    stars: int = betterproto.int32_field(5)
    age: float = betterproto.float_field(6)
    lat_stddev: float = betterproto.float_field(7)
    lon_stddev: float = betterproto.float_field(8)
    l2_df_stars: int = betterproto.int32_field(9)
    pos_type: int = betterproto.int32_field(10)
    c_hash_id: int = betterproto.int64_field(11)
    pos_level: int = betterproto.int32_field(12)


@dataclass(eq=False, repr=False)
class NavBorderDataGetAck(betterproto.Message):
    job_id: int = betterproto.int32_field(1)
    current_frame: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class NavObstiBorderDataGet(betterproto.Message):
    obstacle_index: int = betterproto.int32_field(1)
    current_frame: int = betterproto.int32_field(2)
    obstacles_len: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class NavObstiBorderDataGetAck(betterproto.Message):
    obstacle_index: int = betterproto.int32_field(1)
    current_frame: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class NavCHlLineData(betterproto.Message):
    start_job_ri: int = betterproto.int32_field(1)
    end_job_ri: int = betterproto.int32_field(2)
    current_frame: int = betterproto.int32_field(3)
    channel_line_len: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class NavCHlLineDataAck(betterproto.Message):
    start_job_ri: int = betterproto.int32_field(1)
    end_job_ri: int = betterproto.int32_field(2)
    current_frame: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class NavTaskInfo(betterproto.Message):
    area: int = betterproto.int32_field(1)
    time: int = betterproto.int32_field(2)
    all_frame: int = betterproto.int32_field(3)
    current_frame: int = betterproto.int32_field(4)
    pathlen: int = betterproto.int32_field(5)
    dc: list["CommDataCouple"] = betterproto.message_field(6)


@dataclass(eq=False, repr=False)
class NavBorderDataGet(betterproto.Message):
    job_id: int = betterproto.int32_field(1)
    current_frame: int = betterproto.int32_field(2)
    border_len: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class NavOptLineUp(betterproto.Message):
    start_job_ri: int = betterproto.int32_field(1)
    end_job_ri: int = betterproto.int32_field(2)
    all_frame: int = betterproto.int32_field(3)
    current_frame: int = betterproto.int32_field(4)
    channel_data_len: int = betterproto.int32_field(5)
    dc: list["CommDataCouple"] = betterproto.message_field(6)


@dataclass(eq=False, repr=False)
class NavOptiBorderInfo(betterproto.Message):
    job_id: int = betterproto.int32_field(1)
    all_frame: int = betterproto.int32_field(2)
    current_frame: int = betterproto.int32_field(3)
    border_data_len: int = betterproto.int32_field(4)
    dc: list["CommDataCouple"] = betterproto.message_field(5)


@dataclass(eq=False, repr=False)
class NavOptObsInfo(betterproto.Message):
    obstacle_id: int = betterproto.int32_field(1)
    all_frame: int = betterproto.int32_field(2)
    current_frame: int = betterproto.int32_field(3)
    obstacle_data_len: int = betterproto.int32_field(4)
    dc: list["CommDataCouple"] = betterproto.message_field(5)


@dataclass(eq=False, repr=False)
class NavStartJob(betterproto.Message):
    job_id: int = betterproto.int64_field(1)
    job_ver: int = betterproto.int32_field(2)
    job_mode: int = betterproto.int32_field(3)
    rain_tactics: int = betterproto.int32_field(4)
    knife_height: int = betterproto.int32_field(5)
    speed: float = betterproto.float_field(6)
    channel_width: int = betterproto.int32_field(7)
    ultra_wave: int = betterproto.int32_field(8)
    channel_mode: int = betterproto.int32_field(9)


@dataclass(eq=False, repr=False)
class NavTaskProgress(betterproto.Message):
    task_progress: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class NavResFrame(betterproto.Message):
    frameid: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class NavGetHashList(betterproto.Message):
    pver: int = betterproto.int32_field(1)
    sub_cmd: int = betterproto.int32_field(2)
    total_frame: int = betterproto.int32_field(3)
    current_frame: int = betterproto.int32_field(4)
    data_hash: int = betterproto.fixed64_field(5)
    reserved: str = betterproto.string_field(6)


@dataclass(eq=False, repr=False)
class NavGetHashListAck(betterproto.Message):
    pver: int = betterproto.int32_field(1)
    sub_cmd: int = betterproto.int32_field(2)
    total_frame: int = betterproto.int32_field(3)
    current_frame: int = betterproto.int32_field(4)
    data_hash: int = betterproto.fixed64_field(5)
    hash_len: int = betterproto.int32_field(6)
    reserved: str = betterproto.string_field(7)
    result: int = betterproto.int32_field(8)
    data_couple: list[int] = betterproto.int64_field(13)


@dataclass(eq=False, repr=False)
class NavGetCommData(betterproto.Message):
    pver: int = betterproto.int32_field(1)
    sub_cmd: int = betterproto.int32_field(2)
    action: int = betterproto.int32_field(3)
    type: int = betterproto.int32_field(4)
    hash: int = betterproto.int64_field(5)
    paternal_hash_a: int = betterproto.int64_field(6)
    paternal_hash_b: int = betterproto.int64_field(7)
    total_frame: int = betterproto.int32_field(8)
    current_frame: int = betterproto.int32_field(9)
    data_hash: int = betterproto.fixed64_field(10)
    reserved: str = betterproto.string_field(11)


@dataclass(eq=False, repr=False)
class AreaLabel(betterproto.Message):
    label: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class NavGetCommDataAck(betterproto.Message):
    pver: int = betterproto.int32_field(1)
    sub_cmd: int = betterproto.int32_field(2)
    result: int = betterproto.int32_field(3)
    action: int = betterproto.int32_field(4)
    type: int = betterproto.int32_field(5)
    hash: int = betterproto.fixed64_field(6)
    paternal_hash_a: int = betterproto.fixed64_field(7)
    paternal_hash_b: int = betterproto.fixed64_field(8)
    total_frame: int = betterproto.int32_field(9)
    current_frame: int = betterproto.int32_field(10)
    data_hash: int = betterproto.fixed64_field(11)
    data_len: int = betterproto.int32_field(12)
    data_couple: list["CommDataCouple"] = betterproto.message_field(13)
    reserved: str = betterproto.string_field(14)
    area_label: "AreaLabel" = betterproto.message_field(15)


@dataclass(eq=False, repr=False)
class NavReqCoverPath(betterproto.Message):
    pver: int = betterproto.int32_field(1)
    job_id: int = betterproto.fixed64_field(2)
    job_ver: int = betterproto.int32_field(3)
    job_mode: int = betterproto.int32_field(4)
    sub_cmd: int = betterproto.int32_field(5)
    edge_mode: int = betterproto.int32_field(6)
    knife_height: int = betterproto.int32_field(7)
    channel_width: int = betterproto.int32_field(8)
    ultra_wave: int = betterproto.int32_field(9)
    channel_mode: int = betterproto.int32_field(10)
    toward: int = betterproto.int32_field(11)
    speed: float = betterproto.float_field(12)
    zone_hashs: list[int] = betterproto.fixed64_field(13)
    path_hash: int = betterproto.fixed64_field(14)
    reserved: str = betterproto.string_field(15)
    result: int = betterproto.int32_field(16)
    toward_mode: int = betterproto.int32_field(17)
    toward_included_angle: int = betterproto.int32_field(18)


@dataclass(eq=False, repr=False)
class NavUploadZigZagResult(betterproto.Message):
    pver: int = betterproto.int32_field(1)
    job_id: int = betterproto.fixed64_field(2)
    job_ver: int = betterproto.int32_field(3)
    result: int = betterproto.int32_field(4)
    area: int = betterproto.int32_field(5)
    time: int = betterproto.int32_field(6)
    total_zone_num: int = betterproto.int32_field(7)
    current_zone_path_num: int = betterproto.int32_field(8)
    current_zone_path_id: int = betterproto.int32_field(9)
    current_zone: int = betterproto.int32_field(10)
    current_hash: int = betterproto.fixed64_field(11)
    total_frame: int = betterproto.int32_field(12)
    current_frame: int = betterproto.int32_field(13)
    channel_mode: int = betterproto.int32_field(14)
    channel_mode_id: int = betterproto.int32_field(15)
    data_hash: int = betterproto.fixed64_field(16)
    data_len: int = betterproto.int32_field(17)
    reserved: str = betterproto.string_field(18)
    data_couple: list["CommDataCouple"] = betterproto.message_field(19)
    sub_cmd: int = betterproto.int32_field(20)


@dataclass(eq=False, repr=False)
class NavUploadZigZagResultAck(betterproto.Message):
    pver: int = betterproto.int32_field(1)
    current_zone: int = betterproto.int32_field(2)
    current_hash: int = betterproto.fixed64_field(3)
    total_frame: int = betterproto.int32_field(4)
    current_frame: int = betterproto.int32_field(5)
    data_hash: int = betterproto.fixed64_field(6)
    reserved: str = betterproto.string_field(7)
    sub_cmd: int = betterproto.int32_field(8)


@dataclass(eq=False, repr=False)
class NavTaskCtrl(betterproto.Message):
    type: int = betterproto.int32_field(1)
    action: int = betterproto.int32_field(2)
    result: int = betterproto.int32_field(3)
    reserved: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class NavTaskIdRw(betterproto.Message):
    pver: int = betterproto.int32_field(1)
    sub_cmd: int = betterproto.int32_field(2)
    task_name: str = betterproto.string_field(3)
    task_id: str = betterproto.string_field(4)
    result: int = betterproto.int32_field(5)
    reserved: str = betterproto.string_field(6)


@dataclass(eq=False, repr=False)
class NavSysHashOverview(betterproto.Message):
    commonhash_overview: int = betterproto.fixed64_field(1)
    path_hash_overview: int = betterproto.fixed64_field(2)


@dataclass(eq=False, repr=False)
class NavTaskBreakPoint(betterproto.Message):
    x: float = betterproto.float_field(1)
    y: float = betterproto.float_field(2)
    toward: int = betterproto.int32_field(3)
    flag: int = betterproto.int32_field(4)
    action: int = betterproto.int32_field(5)
    zone_hash: int = betterproto.fixed64_field(6)


@dataclass(eq=False, repr=False)
class NavPlanJobSet(betterproto.Message):
    pver: int = betterproto.int32_field(1)
    sub_cmd: int = betterproto.int32_field(2)
    area: int = betterproto.int32_field(3)
    work_time: int = betterproto.int32_field(4)
    version: str = betterproto.string_field(5)
    id: str = betterproto.string_field(6)
    user_id: str = betterproto.string_field(7)
    device_id: str = betterproto.string_field(8)
    plan_id: str = betterproto.string_field(9)
    task_id: str = betterproto.string_field(10)
    job_id: str = betterproto.string_field(11)
    start_time: str = betterproto.string_field(12)
    end_time: str = betterproto.string_field(13)
    week: int = betterproto.int32_field(14)
    knife_height: int = betterproto.int32_field(15)
    model: int = betterproto.int32_field(16)
    edge_mode: int = betterproto.int32_field(17)
    required_time: int = betterproto.int32_field(18)
    route_angle: int = betterproto.int32_field(19)
    route_model: int = betterproto.int32_field(20)
    route_spacing: int = betterproto.int32_field(21)
    ultrasonic_barrier: int = betterproto.int32_field(22)
    total_plan_num: int = betterproto.int32_field(23)
    plan_index: int = betterproto.int32_field(24)
    result: int = betterproto.int32_field(25)
    speed: float = betterproto.float_field(26)
    task_name: str = betterproto.string_field(27)
    job_name: str = betterproto.string_field(28)
    zone_hashs: list[int] = betterproto.fixed64_field(29)
    reserved: str = betterproto.string_field(30)
    start_date: str = betterproto.string_field(31)
    end_date: str = betterproto.string_field(32)
    trigger_type: int = betterproto.int32_field(33)
    day: int = betterproto.int32_field(34)
    weeks: list[int] = betterproto.fixed32_field(35)
    remained_seconds: int = betterproto.int64_field(36)
    toward_mode: int = betterproto.int32_field(37)
    toward_included_angle: int = betterproto.int32_field(38)


@dataclass(eq=False, repr=False)
class NavUnableTimeSet(betterproto.Message):
    sub_cmd: int = betterproto.int32_field(1)
    device_id: str = betterproto.string_field(2)
    unable_start_time: str = betterproto.string_field(3)
    unable_end_time: str = betterproto.string_field(4)
    result: int = betterproto.int32_field(5)
    reserved: str = betterproto.string_field(6)


@dataclass(eq=False, repr=False)
class ChargePileType(betterproto.Message):
    toward: int = betterproto.int32_field(1)
    x: float = betterproto.float_field(2)
    y: float = betterproto.float_field(3)


@dataclass(eq=False, repr=False)
class SimulationCmdData(betterproto.Message):
    sub_cmd: int = betterproto.int32_field(1)
    param_id: int = betterproto.int32_field(2)
    param_value: list[int] = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class WorkReportUpdateCmd(betterproto.Message):
    sub_cmd: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class WorkReportUpdateAck(betterproto.Message):
    update_flag: bool = betterproto.bool_field(1)
    info_num: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class WorkReportCmdData(betterproto.Message):
    sub_cmd: int = betterproto.int32_field(1)
    get_info_num: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class WorkReportInfoAck(betterproto.Message):
    interrupt_flag: bool = betterproto.bool_field(1)
    start_work_time: int = betterproto.int64_field(2)
    end_work_time: int = betterproto.int64_field(3)
    work_time_used: int = betterproto.int32_field(4)
    work_ares: float = betterproto.double_field(5)
    work_progress: int = betterproto.int32_field(6)
    height_of_knife: int = betterproto.int32_field(7)
    work_type: int = betterproto.int32_field(8)
    work_result: int = betterproto.int32_field(9)
    total_ack_num: int = betterproto.int32_field(10)
    current_ack_num: int = betterproto.int32_field(11)


@dataclass(eq=False, repr=False)
class AppRequestCoverPathsT(betterproto.Message):
    pver: int = betterproto.int32_field(1)
    sub_cmd: int = betterproto.int32_field(2)
    total_frame: int = betterproto.int32_field(3)
    current_frame: int = betterproto.int32_field(4)
    data_hash: int = betterproto.fixed64_field(5)
    transaction_id: int = betterproto.int64_field(6)
    reserved: list[int] = betterproto.int64_field(7)
    hash_list: list[int] = betterproto.fixed64_field(8)


@dataclass(eq=False, repr=False)
class CoverPathPacketT(betterproto.Message):
    path_hash: int = betterproto.fixed64_field(1)
    path_type: int = betterproto.int32_field(2)
    path_total: int = betterproto.int32_field(3)
    path_cur: int = betterproto.int32_field(4)
    zone_hash: int = betterproto.fixed64_field(5)
    data_couple: list["CommDataCouple"] = betterproto.message_field(6)


@dataclass(eq=False, repr=False)
class CoverPathUploadT(betterproto.Message):
    pver: int = betterproto.int32_field(1)
    result: int = betterproto.int32_field(2)
    sub_cmd: int = betterproto.int32_field(3)
    area: int = betterproto.int32_field(4)
    time: int = betterproto.int32_field(5)
    total_frame: int = betterproto.int32_field(6)
    current_frame: int = betterproto.int32_field(7)
    total_path_num: int = betterproto.int32_field(8)
    vaild_path_num: int = betterproto.int32_field(9)
    data_hash: int = betterproto.fixed64_field(10)
    transaction_id: int = betterproto.int64_field(11)
    reserved: list[int] = betterproto.int64_field(12)
    data_len: int = betterproto.int32_field(13)
    path_packets: list["CoverPathPacketT"] = betterproto.message_field(14)


@dataclass(eq=False, repr=False)
class ZoneStartPrecentT(betterproto.Message):
    data_hash: int = betterproto.fixed64_field(1)
    x: float = betterproto.float_field(2)
    y: float = betterproto.float_field(3)
    index: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class VisionCtrlMsg(betterproto.Message):
    type: int = betterproto.int32_field(1)
    cmd: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class NavSysParamMsg(betterproto.Message):
    rw: int = betterproto.int32_field(1)
    id: int = betterproto.int32_field(2)
    context: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class NavPlanTaskExecute(betterproto.Message):
    sub_cmd: int = betterproto.int32_field(1)
    id: str = betterproto.string_field(2)
    name: str = betterproto.string_field(3)
    result: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class CostmapT(betterproto.Message):
    width: int = betterproto.int32_field(1)
    height: int = betterproto.int32_field(2)
    center_x: float = betterproto.float_field(3)
    center_y: float = betterproto.float_field(4)
    yaw: float = betterproto.float_field(5)
    res: float = betterproto.float_field(6)
    costmap: list[int] = betterproto.int32_field(7)


@dataclass(eq=False, repr=False)
class PlanTaskNameIdT(betterproto.Message):
    id: str = betterproto.string_field(1)
    name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class NavGetAllPlanTask(betterproto.Message):
    tasks: list["PlanTaskNameIdT"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class NavTaskCtrlAck(betterproto.Message):
    """Define the NavTaskCtrlAck message"""

    type: int = betterproto.int32_field(1)
    action: int = betterproto.int32_field(2)
    result: int = betterproto.int32_field(3)
    nav_state: int = betterproto.int32_field(4)
    reserved: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class NavMapNameMsg(betterproto.Message):
    rw: int = betterproto.int32_field(1)
    hash: int = betterproto.fixed64_field(2)
    name: str = betterproto.string_field(3)
    result: int = betterproto.int32_field(4)
    device_id: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class SvgMessageT(betterproto.Message):
    x_move: float = betterproto.double_field(1)
    y_move: float = betterproto.double_field(2)
    scale: float = betterproto.double_field(3)
    rotate: float = betterproto.double_field(4)
    base_width_m: float = betterproto.double_field(5)
    base_width_pix: int = betterproto.int32_field(7)
    base_height_m: float = betterproto.double_field(6)
    base_height_pix: int = betterproto.int32_field(8)
    data_count: int = betterproto.int32_field(12)
    hide_svg: bool = betterproto.bool_field(13)
    name_count: int = betterproto.int32_field(11)
    svg_file_name: str = betterproto.string_field(9)
    svg_file_data: str = betterproto.string_field(10)


@dataclass(eq=False, repr=False)
class SvgMessageAckT(betterproto.Message):
    pver: int = betterproto.int32_field(1)
    sub_cmd: int = betterproto.int32_field(2)
    total_frame: int = betterproto.int32_field(3)
    current_frame: int = betterproto.int32_field(4)
    data_hash: int = betterproto.fixed64_field(5)
    paternal_hash_a: int = betterproto.fixed64_field(6)
    type: int = betterproto.int32_field(7)
    result: int = betterproto.int32_field(8)
    svg_message: "SvgMessageT" = betterproto.message_field(9)


@dataclass(eq=False, repr=False)
class AreaHashName(betterproto.Message):
    name: str = betterproto.string_field(2)
    hash: int = betterproto.fixed64_field(1)


@dataclass(eq=False, repr=False)
class AppGetAllAreaHashName(betterproto.Message):
    device_id: str = betterproto.string_field(1)
    hashnames: list["AreaHashName"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class MctlNav(betterproto.Message):
    toapp_lat_up: "NavLatLonUp" = betterproto.message_field(
        1, group="SubNavMsg"
    )
    toapp_pos_up: "NavPosUp" = betterproto.message_field(
        2, group="SubNavMsg"
    )
    todev_chl_line_data: "NavCHlLineData" = betterproto.message_field(
        3, group="SubNavMsg"
    )
    toapp_task_info: "NavTaskInfo" = betterproto.message_field(
        4, group="SubNavMsg"
    )
    toapp_opt_line_up: "NavOptLineUp" = betterproto.message_field(
        5, group="SubNavMsg"
    )
    toapp_opt_border_info: "NavOptiBorderInfo" = betterproto.message_field(
        6, group="SubNavMsg"
    )
    toapp_opt_obs_info: "NavOptObsInfo" = betterproto.message_field(
        7, group="SubNavMsg"
    )
    todev_task_info_ack: "NavResFrame" = betterproto.message_field(
        8, group="SubNavMsg"
    )
    todev_opt_border_info_ack: "NavResFrame" = betterproto.message_field(
        9, group="SubNavMsg"
    )
    todev_opt_obs_info_ack: "NavResFrame" = betterproto.message_field(
        10, group="SubNavMsg"
    )
    todev_opt_line_up_ack: "NavResFrame" = betterproto.message_field(
        11, group="SubNavMsg"
    )
    toapp_chgpileto: "ChargePileType" = betterproto.message_field(
        12, group="SubNavMsg"
    )
    todev_sustask: int = betterproto.int32_field(
        13, group="SubNavMsg"
    )
    todev_rechgcmd: int = betterproto.int32_field(
        14, group="SubNavMsg"
    )
    todev_edgecmd: int = betterproto.int32_field(
        15, group="SubNavMsg"
    )
    todev_draw_border: int = betterproto.int32_field(
        16, group="SubNavMsg"
    )
    todev_draw_border_end: int = betterproto.int32_field(
        17, group="SubNavMsg"
    )
    todev_draw_obs: int = betterproto.int32_field(
        18, group="SubNavMsg"
    )
    todev_draw_obs_end: int = betterproto.int32_field(
        19, group="SubNavMsg"
    )
    todev_chl_line: int = betterproto.int32_field(
        20, group="SubNavMsg"
    )
    todev_chl_line_end: int = betterproto.int32_field(
        21, group="SubNavMsg"
    )
    todev_save_task: int = betterproto.int32_field(
        22, group="SubNavMsg"
    )
    todev_cancel_suscmd: int = betterproto.int32_field(
        23, group="SubNavMsg"
    )
    todev_reset_chg_pile: int = betterproto.int32_field(
        24, group="SubNavMsg"
    )
    todev_cancel_draw_cmd: int = betterproto.int32_field(
        25, group="SubNavMsg"
    )
    todev_one_touch_leave_pile: int = betterproto.int32_field(
        26, group="SubNavMsg"
    )
    todev_mow_task: "NavStartJob" = betterproto.message_field(
        27, group="SubNavMsg"
    )
    toapp_bstate: "NavBorderState" = betterproto.message_field(
        28, group="SubNavMsg"
    )
    todev_lat_up_ack: int = betterproto.int32_field(
        29, group="SubNavMsg"
    )
    todev_gethash: "NavGetHashList" = betterproto.message_field(
        30, group="SubNavMsg"
    )
    toapp_gethash_ack: "NavGetHashListAck" = betterproto.message_field(
        31, group="SubNavMsg"
    )
    todev_get_commondata: "NavGetCommData" = betterproto.message_field(
        32, group="SubNavMsg"
    )
    toapp_get_commondata_ack: "NavGetCommDataAck" = betterproto.message_field(
        33, group="SubNavMsg"
    )
    bidire_reqconver_path: "NavReqCoverPath" = betterproto.message_field(
        34, group="SubNavMsg"
    )
    toapp_zigzag: "NavUploadZigZagResult" = betterproto.message_field(
        35, group="SubNavMsg"
    )
    todev_zigzag_ack: "NavUploadZigZagResultAck" = betterproto.message_field(
        36, group="SubNavMsg"
    )
    todev_taskctrl: "NavTaskCtrl" = betterproto.message_field(
        37, group="SubNavMsg"
    )
    bidire_taskid: "NavTaskIdRw" = betterproto.message_field(
        38, group="SubNavMsg"
    )
    toapp_bp: "NavTaskBreakPoint" = betterproto.message_field(
        39, group="SubNavMsg"
    )
    todev_planjob_set: "NavPlanJobSet" = betterproto.message_field(
        40, group="SubNavMsg"
    )
    todev_unable_time_set: "NavUnableTimeSet" = betterproto.message_field(
        41, group="SubNavMsg"
    )
    simulation_cmd: "SimulationCmdData" = betterproto.message_field(
        42, group="SubNavMsg"
    )
    todev_work_report_update_cmd: "WorkReportUpdateCmd" = (
        betterproto.message_field(43, group="SubNavMsg")
    )
    toapp_work_report_update_ack: "WorkReportUpdateAck" = (
        betterproto.message_field(44, group="SubNavMsg")
    )
    todev_work_report_cmd: "WorkReportCmdData" = betterproto.message_field(
        45, group="SubNavMsg"
    )
    toapp_work_report_ack: "WorkReportInfoAck" = betterproto.message_field(
        46, group="SubNavMsg"
    )
    toapp_work_report_upload: "WorkReportInfoAck" = betterproto.message_field(
        47, group="SubNavMsg"
    )
    app_request_cover_paths: "AppRequestCoverPathsT" = (
        betterproto.message_field(48, group="SubNavMsg")
    )
    cover_path_upload: "CoverPathUploadT" = betterproto.message_field(
        49, group="SubNavMsg"
    )
    zone_start_precent: "ZoneStartPrecentT" = betterproto.message_field(
        50, group="SubNavMsg"
    )
    vision_ctrl: "VisionCtrlMsg" = betterproto.message_field(
        51, group="SubNavMsg"
    )
    nav_sys_param_cmd: "NavSysParamMsg" = betterproto.message_field(
        52, group="SubNavMsg"
    )
    plan_task_execute: "NavPlanTaskExecute" = betterproto.message_field(
        53, group="SubNavMsg"
    )
    toapp_costmap: "CostmapT" = betterproto.message_field(
        54, group="SubNavMsg"
    )
    plan_task_name_id: "PlanTaskNameIdT" = betterproto.message_field(
        55, group="SubNavMsg"
    )
    all_plan_task: "NavGetAllPlanTask" = betterproto.message_field(
        56, group="SubNavMsg"
    )
    todev_taskctrl_ack: "NavTaskCtrlAck" = betterproto.message_field(
        57, group="SubNavMsg"
    )
    toapp_map_name_msg: "NavMapNameMsg" = betterproto.message_field(
        58, group="SubNavMsg"
    )
    todev_svg_msg: "SvgMessageAckT" = betterproto.message_field(
        59, group="SubNavMsg"
    )
    toapp_svg_msg: "SvgMessageAckT" = betterproto.message_field(
        60, group="SubNavMsg"
    )
    toapp_all_hash_name: "AppGetAllAreaHashName" = betterproto.message_field(
        61, group="SubNavMsg"
    )



@dataclass(eq=False, repr=False)
class SysBatUp(betterproto.Message):
    bat_val: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class SysWorkState(betterproto.Message):
    device_state: int = betterproto.int32_field(1)
    charge_state: int = betterproto.int32_field(2)
    cm_hash: int = betterproto.int64_field(3)
    path_hash: int = betterproto.int64_field(4)


@dataclass(eq=False, repr=False)
class SysSetTimeZone(betterproto.Message):
    time_stamp: int = betterproto.int32_field(1)
    time_area: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class SysSetDateTime(betterproto.Message):
    year: int = betterproto.int32_field(1)
    month: int = betterproto.int32_field(2)
    date: int = betterproto.int32_field(3)
    week: int = betterproto.int32_field(4)
    hours: int = betterproto.int32_field(5)
    minutes: int = betterproto.int32_field(6)
    seconds: int = betterproto.int32_field(7)
    time_zone: int = betterproto.int32_field(8)
    daylight: int = betterproto.int32_field(9)


@dataclass(eq=False, repr=False)
class SysJobPlan(betterproto.Message):
    job_id: int = betterproto.int64_field(1)
    job_mode: int = betterproto.int32_field(2)
    rain_tactics: int = betterproto.int32_field(3)
    knife_height: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class SysDevErrCode(betterproto.Message):
    error_code: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class SysBoardType(betterproto.Message):
    board_type: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class SysSwVersion(betterproto.Message):
    board_type: int = betterproto.int32_field(1)
    version_len: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class SysDelJobPlan(betterproto.Message):
    device_id: str = betterproto.string_field(1)
    plan_id: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class SysJobPlanTime(betterproto.Message):
    plan_id: int = betterproto.int64_field(1)
    start_job_time: int = betterproto.int32_field(2)
    end_job_time: int = betterproto.int32_field(3)
    time_in_day: int = betterproto.int32_field(4)
    job_plan_mode: int = betterproto.int32_field(5)
    job_plan_enable: int = betterproto.int32_field(6)
    week_day: list[int] = betterproto.int32_field(7)
    time_in_week_day: list[int] = betterproto.int32_field(8)
    every_day: int = betterproto.int32_field(9)
    job_plan: "SysJobPlan" = betterproto.message_field(10)


@dataclass(eq=False, repr=False)
class SysMowInfo(betterproto.Message):
    device_state: int = betterproto.int32_field(1)
    bat_val: int = betterproto.int32_field(2)
    knife_height: int = betterproto.int32_field(3)
    rtk_status: int = betterproto.int32_field(4)
    rtk_stars: int = betterproto.int32_field(5)


@dataclass(eq=False, repr=False)
class SysOptiLineAck(betterproto.Message):
    respones_cmd: int = betterproto.int32_field(1)
    current_frame: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class SysCommCmd(betterproto.Message):
    rw: int = betterproto.int32_field(1)
    id: int = betterproto.int32_field(2)
    context: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class SysUploadFileProgress(betterproto.Message):
    biz_id: str = betterproto.string_field(1)
    result: int = betterproto.int32_field(2)
    progress: int = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class SysErrorCode(betterproto.Message):
    code_no: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class SysBorder(betterproto.Message):
    borderval: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class SysPlanJobStatus(betterproto.Message):
    planjob_status: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class SysKnifeControl(betterproto.Message):
    knife_status: int = betterproto.int32_field(1)
    knife_height: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class SysResetSystemStatus(betterproto.Message):
    reset_staus: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class TimeCtrlLight(betterproto.Message):
    operate: int = betterproto.int32_field(1)
    enable: int = betterproto.int32_field(2)
    start_hour: int = betterproto.int32_field(3)
    start_min: int = betterproto.int32_field(4)
    end_hour: int = betterproto.int32_field(5)
    end_min: int = betterproto.int32_field(6)
    action: int = betterproto.int32_field(7)


@dataclass(eq=False, repr=False)
class VisionPointMsg(betterproto.Message):
    x: float = betterproto.float_field(1)
    y: float = betterproto.float_field(2)
    z: float = betterproto.float_field(3)


@dataclass(eq=False, repr=False)
class VisionPointInfoMsg(betterproto.Message):
    label: int = betterproto.int32_field(1)
    num: int = betterproto.int32_field(2)
    vision_point: list["VisionPointMsg"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class VioToAppInfoMsg(betterproto.Message):
    x: float = betterproto.double_field(1)
    y: float = betterproto.double_field(2)
    heading: float = betterproto.double_field(3)
    vio_state: int = betterproto.int32_field(4)
    brightness: int = betterproto.int32_field(5)
    detect_feature_num: int = betterproto.int32_field(6)
    track_feature_num: int = betterproto.int32_field(7)


@dataclass(eq=False, repr=False)
class VisionStatisticMsg(betterproto.Message):
    mean: float = betterproto.float_field(1)
    var: float = betterproto.float_field(2)


@dataclass(eq=False, repr=False)
class VisionStatisticInfoMsg(betterproto.Message):
    timestamp: float = betterproto.double_field(1)
    num: int = betterproto.int32_field(2)
    vision_statistics: list["VisionStatisticMsg"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class SystemRapidStateTunnelMsg(betterproto.Message):
    rapid_state_data: list[int] = betterproto.int64_field(1)
    vision_point_info: list["VisionPointInfoMsg"] = betterproto.message_field(2)
    vio_to_app_info: "VioToAppInfoMsg" = betterproto.message_field(3)
    vision_statistic_info: "VisionStatisticInfoMsg" = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class SystemTardStateTunnelMsg(betterproto.Message):
    tard_state_data: list[int] = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class SystemUpdateBufMsg(betterproto.Message):
    update_buf_data: list[int] = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class SysOffChipFlash(betterproto.Message):
    op: "Operation" = betterproto.enum_field(1)
    id: "OffPartId" = betterproto.enum_field(2)
    start_addr: int = betterproto.uint32_field(3)
    offset: int = betterproto.uint32_field(4)
    length: int = betterproto.int32_field(5)
    data: bytes = betterproto.bytes_field(6)
    code: int = betterproto.int32_field(7)
    msg: str = betterproto.string_field(8)


@dataclass(eq=False, repr=False)
class SystemTmpCycleTxMsg(betterproto.Message):
    cycle_tx_data: list[int] = betterproto.int64_field(1)


@dataclass(eq=False, repr=False)
class LoraCfgReq(betterproto.Message):
    op: int = betterproto.int32_field(1)
    cfg: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class LoraCfgRsp(betterproto.Message):
    result: int = betterproto.int32_field(1)
    op: int = betterproto.int32_field(2)
    cfg: str = betterproto.string_field(3)
    fac_cfg: bytes = betterproto.bytes_field(4)


@dataclass(eq=False, repr=False)
class ModFwInfo(betterproto.Message):
    type: int = betterproto.int32_field(1)
    identify: str = betterproto.string_field(2)
    version: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class DeviceFwInfo(betterproto.Message):
    result: int = betterproto.int32_field(1)
    version: str = betterproto.string_field(2)
    mod: list["ModFwInfo"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class MowToAppInfoT(betterproto.Message):
    type: int = betterproto.int32_field(1)
    cmd: int = betterproto.int32_field(2)
    mow_data: list[int] = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class DeviceProductTypeInfoT(betterproto.Message):
    result: int = betterproto.int32_field(1)
    main_product_type: str = betterproto.string_field(2)
    sub_product_type: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class QcAppTestExcept(betterproto.Message):
    except_type: str = betterproto.string_field(1)
    conditions: list["QcAppTestConditions"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class QcAppTestConditions(betterproto.Message):
    cond_type: str = betterproto.string_field(1)
    int_val: int = betterproto.int32_field(2)
    float_val: float = betterproto.float_field(3)
    double_val: float = betterproto.double_field(4)
    string_val: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class MowToAppQctoolsInfoT(betterproto.Message):
    type: "QcAppTestId" = betterproto.enum_field(1)
    time_of_duration: int = betterproto.int32_field(2)
    result: int = betterproto.int32_field(3)
    result_details: str = betterproto.string_field(4)
    except_: list["QcAppTestExcept"] = betterproto.message_field(5)


@dataclass(eq=False, repr=False)
class MCtrlSimulationCmdData(betterproto.Message):
    sub_cmd: int = betterproto.int32_field(1)
    param_id: int = betterproto.int32_field(2)
    param_value: list[int] = betterproto.int32_field(3)


@dataclass(eq=False, repr=False)
class RptLora(betterproto.Message):
    pair_code_scan: int = betterproto.int32_field(1)
    pair_code_channel: int = betterproto.int32_field(2)
    pair_code_locid: int = betterproto.int32_field(3)
    pair_code_netid: int = betterproto.int32_field(4)
    lora_connection_status: int = betterproto.int32_field(5)


@dataclass(eq=False, repr=False)
class MqttRtkConnect(betterproto.Message):
    rtk_switch: int = betterproto.int32_field(1)
    rtk_channel: int = betterproto.int32_field(2)
    rtk_base_num: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class RptRtk(betterproto.Message):
    status: int = betterproto.int32_field(1)
    pos_level: int = betterproto.int32_field(2)
    gps_stars: int = betterproto.int32_field(3)
    age: int = betterproto.int32_field(4)
    lat_std: int = betterproto.int32_field(5)
    lon_std: int = betterproto.int32_field(6)
    l2_stars: int = betterproto.int32_field(7)
    dis_status: int = betterproto.int64_field(8)
    top4_total_mean: int = betterproto.int64_field(9)
    co_view_stars: int = betterproto.int32_field(10)
    reset: int = betterproto.int32_field(11)
    lora_info: "RptLora" = betterproto.message_field(12)
    mqtt_rtk_info: "MqttRtkConnect" = betterproto.message_field(13)


@dataclass(eq=False, repr=False)
class RptDevLocation(betterproto.Message):
    real_pos_x: int = betterproto.int32_field(1)
    real_pos_y: int = betterproto.int32_field(2)
    real_toward: int = betterproto.int32_field(3)
    pos_type: int = betterproto.int32_field(4)
    zone_hash: int = betterproto.int64_field(5)
    bol_hash: int = betterproto.int64_field(6)


@dataclass(eq=False, repr=False)
class VioSurvivalInfoT(betterproto.Message):
    vio_survival_distance: float = betterproto.float_field(1)


@dataclass(eq=False, repr=False)
class CollectorStatusT(betterproto.Message):
    collector_installation_status: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class LockStateT(betterproto.Message):
    lock_state: int = betterproto.uint32_field(1)


@dataclass(eq=False, repr=False)
class RptDevStatus(betterproto.Message):
    sys_status: int = betterproto.int32_field(1)
    charge_state: int = betterproto.int32_field(2)
    battery_val: int = betterproto.int32_field(3)
    sensor_status: int = betterproto.int32_field(4)
    last_status: int = betterproto.int32_field(5)
    sys_time_stamp: int = betterproto.int64_field(6)
    vslam_status: int = betterproto.int32_field(7)
    mnet_info: "MnetInfo" = betterproto.message_field(8)
    vio_survival_info: "VioSurvivalInfoT" = betterproto.message_field(9)
    collector_status: "CollectorStatusT" = betterproto.message_field(10)
    lock_state: "LockStateT" = betterproto.message_field(11)


@dataclass(eq=False, repr=False)
class RptConnectStatus(betterproto.Message):
    connect_type: int = betterproto.int32_field(1)
    ble_rssi: int = betterproto.int32_field(2)
    wifi_rssi: int = betterproto.int32_field(3)
    link_type: int = betterproto.int32_field(4)
    mnet_rssi: int = betterproto.int32_field(5)
    mnet_inet: int = betterproto.int32_field(6)
    used_net: "NetUsedType" = betterproto.enum_field(7)
    mnet_cfg: "MnetCfg" = betterproto.message_field(8)


@dataclass(eq=False, repr=False)
class NavHeadingStateT(betterproto.Message):
    heading_state: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class RptWork(betterproto.Message):
    plan: int = betterproto.int32_field(1)
    path_hash: int = betterproto.int64_field(2)
    progress: int = betterproto.int32_field(3)
    area: int = betterproto.int32_field(4)
    bp_info: int = betterproto.int32_field(5)
    bp_hash: int = betterproto.int64_field(6)
    bp_pos_x: int = betterproto.int32_field(7)
    bp_pos_y: int = betterproto.int32_field(8)
    real_path_num: int = betterproto.int64_field(9)
    path_pos_x: int = betterproto.int32_field(10)
    path_pos_y: int = betterproto.int32_field(11)
    ub_zone_hash: int = betterproto.int64_field(12)
    ub_path_hash: int = betterproto.int64_field(13)
    init_cfg_hash: int = betterproto.int64_field(14)
    ub_ecode_hash: int = betterproto.int64_field(15)
    nav_run_mode: int = betterproto.int32_field(16)
    test_mode_status: int = betterproto.int64_field(17)
    man_run_speed: int = betterproto.int32_field(18)
    nav_edit_status: int = betterproto.int32_field(19)
    knife_height: int = betterproto.int32_field(20)
    nav_heading_state: "NavHeadingStateT" = betterproto.message_field(21)


@dataclass(eq=False, repr=False)
class BladeUsed(betterproto.Message):
    blade_used_time: int = betterproto.int32_field(1)
    blade_used_warn_time: int = betterproto.int32_field(2)



@dataclass(eq=False, repr=False)
class RptMaintain(betterproto.Message):
    mileage: int = betterproto.int64_field(1)
    work_time: int = betterproto.int32_field(2)
    bat_cycles: int = betterproto.int32_field(3)
    blade_used_time: BladeUsed = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class ReportInfoCfg(betterproto.Message):
    act: "RptAct" = betterproto.enum_field(1)
    timeout: int = betterproto.int32_field(2)
    period: int = betterproto.int32_field(3)
    no_change_period: int = betterproto.int32_field(4)
    count: int = betterproto.int32_field(5)
    sub: list["RptInfoType"] = betterproto.enum_field(6)


@dataclass(eq=False, repr=False)
class ReportInfoData(betterproto.Message):
    connect: "RptConnectStatus" = betterproto.message_field(1)
    dev: "RptDevStatus" = betterproto.message_field(2)
    rtk: "RptRtk" = betterproto.message_field(3)
    locations: list["RptDevLocation"] = betterproto.message_field(4)
    work: "RptWork" = betterproto.message_field(5)
    fw_info: "DeviceFwInfo" = betterproto.message_field(6)
    maintain: "RptMaintain" = betterproto.message_field(7)
    vision_point_info: list["VisionPointInfoMsg"] = betterproto.message_field(8)
    vio_to_app_info: "VioToAppInfoMsg" = betterproto.message_field(9)
    vision_statistic_info: "VisionStatisticInfoMsg" = betterproto.message_field(10)


@dataclass(eq=False, repr=False)
class MctlSys(betterproto.Message):
    toapp_batinfo: "SysBatUp" = betterproto.message_field(
        1, group="SubSysMsg"
    )
    toapp_work_state: "SysWorkState" = betterproto.message_field(
        2, group="SubSysMsg"
    )
    todev_time_zone: "SysSetTimeZone" = betterproto.message_field(
        3, group="SubSysMsg"
    )
    todev_data_time: "SysSetDateTime" = betterproto.message_field(
        4, group="SubSysMsg"
    )
    job_plan: "SysJobPlan" = betterproto.message_field(
        6, group="SubSysMsg"
    )
    toapp_err_code: "SysDevErrCode" = betterproto.message_field(
        7, group="SubSysMsg"
    )
    todev_job_plan_time: "SysJobPlanTime" = betterproto.message_field(
        10, group="SubSysMsg"
    )
    toapp_mow_info: "SysMowInfo" = betterproto.message_field(
        11, group="SubSysMsg"
    )
    bidire_comm_cmd: "SysCommCmd" = betterproto.message_field(
        12, group="SubSysMsg"
    )
    plan_job_del: int = betterproto.int64_field(
        14, group="SubSysMsg"
    )
    border: "SysBorder" = betterproto.message_field(
        15, group="SubSysMsg"
    )
    toapp_plan_status: "SysPlanJobStatus" = betterproto.message_field(
        18, group="SubSysMsg"
    )
    toapp_ul_fprogress: "SysUploadFileProgress" = betterproto.message_field(
        19, group="SubSysMsg"
    )
    todev_deljobplan: "SysDelJobPlan" = betterproto.message_field(
        20, group="SubSysMsg"
    )
    todev_mow_info_up: int = betterproto.int32_field(
        21, group="SubSysMsg"
    )
    todev_knife_ctrl: "SysKnifeControl" = betterproto.message_field(
        22, group="SubSysMsg"
    )
    todev_reset_system: int = betterproto.int32_field(
        23, group="SubSysMsg"
    )
    todev_reset_system_status: "SysResetSystemStatus" = (
        betterproto.message_field(24, group="SubSysMsg")
    )
    system_rapid_state_tunnel: "SystemRapidStateTunnelMsg" = (
        betterproto.message_field(25, group="SubSysMsg")
    )
    system_tard_state_tunnel: "SystemTardStateTunnelMsg" = (
        betterproto.message_field(26, group="SubSysMsg")
    )
    system_update_buf: "SystemUpdateBufMsg" = betterproto.message_field(
        27, group="SubSysMsg"
    )
    todev_time_ctrl_light: "TimeCtrlLight" = betterproto.message_field(
        28, group="SubSysMsg"
    )
    system_tmp_cycle_tx: "SystemTmpCycleTxMsg" = betterproto.message_field(
        29, group="SubSysMsg"
    )
    todev_off_chip_flash: "SysOffChipFlash" = betterproto.message_field(
        30, group="SubSysMsg"
    )
    todev_get_dev_fw_info: int = betterproto.int32_field(
        31, group="SubSysMsg"
    )
    toapp_dev_fw_info: "DeviceFwInfo" = betterproto.message_field(
        32, group="SubSysMsg"
    )
    todev_lora_cfg_req: "LoraCfgReq" = betterproto.message_field(
        33, group="SubSysMsg"
    )
    toapp_lora_cfg_rsp: "LoraCfgRsp" = betterproto.message_field(
        34, group="SubSysMsg"
    )
    mow_to_app_info: "MowToAppInfoT" = betterproto.message_field(
        35, group="SubSysMsg"
    )
    device_product_type_info: "DeviceProductTypeInfoT" = (
        betterproto.message_field(36, group="SubSysMsg")
    )
    mow_to_app_qctools_info: "MowToAppQctoolsInfoT" = (
        betterproto.message_field(37, group="SubSysMsg")
    )
    todev_report_cfg: "ReportInfoCfg" = betterproto.message_field(
        38, group="SubSysMsg"
    )
    toapp_report_data: "ReportInfoData" = betterproto.message_field(
        39, group="SubSysMsg"
    )
    simulation_cmd: "MCtrlSimulationCmdData" = betterproto.message_field(
        42, group="SubSysMsg"
    )



@dataclass(eq=False, repr=False)
class BaseInfo(betterproto.Message):
    dev_version: str = betterproto.string_field(1)
    dev_status: int = betterproto.int32_field(2)
    batt_val: int = betterproto.int32_field(3)
    init_status: int = betterproto.int32_field(4)
    is_tilt: int = betterproto.int32_field(5)


@dataclass(eq=False, repr=False)
class OtaInfo(betterproto.Message):
    otaid: str = betterproto.string_field(1)
    version: str = betterproto.string_field(2)
    progress: int = betterproto.int32_field(3)
    result: int = betterproto.int32_field(4)
    message: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class GetInfoReq(betterproto.Message):
    type: "InfoType" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class GetInfoRsp(betterproto.Message):
    result: int = betterproto.int32_field(1)
    type: "InfoType" = betterproto.enum_field(2)
    base: "BaseInfo" = betterproto.message_field(
        3, group="info"
    )
    ota: "OtaInfo" = betterproto.message_field(4, group="info")



@dataclass(eq=False, repr=False)
class MctlOta(betterproto.Message):
    todev_get_info_req: "GetInfoReq" = betterproto.message_field(
        1, group="SubOtaMsg"
    )
    toapp_get_info_rsp: "GetInfoRsp" = betterproto.message_field(
        2, group="SubOtaMsg"
    )



@dataclass(eq=False, repr=False)
class MulSetAudio(betterproto.Message):
    at_switch: int = betterproto.int32_field(
        1, group="AudioCfg_u"
    )
    au_language: "MulLanguage" = betterproto.enum_field(
        2, group="AudioCfg_u"
    )



@dataclass(eq=False, repr=False)
class MulSetVideo(betterproto.Message):
    position: "MulCameraPosition" = betterproto.enum_field(1)
    vi_switch: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class MulSetVideoAck(betterproto.Message):
    error_code: "MulVideoErrorCode" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class MulAudioCfg(betterproto.Message):
    au_switch: int = betterproto.int32_field(1)
    au_language: "MulLanguage" = betterproto.enum_field(2)


@dataclass(eq=False, repr=False)
class MulSetWiper(betterproto.Message):
    round: int = betterproto.int32_field(1)


@dataclass(eq=False, repr=False)
class MulSetWiperAck(betterproto.Message):
    error_code: "MulWiperErrorCode" = betterproto.enum_field(1)


@dataclass(eq=False, repr=False)
class SocMul(betterproto.Message):
    set_audio: "MulSetAudio" = betterproto.message_field(
        1, group="SubMul"
    )
    audio_cfg: "MulAudioCfg" = betterproto.message_field(
        2, group="SubMul"
    )
    set_video: "MulSetVideo" = betterproto.message_field(
        3, group="SubMul"
    )
    set_video_ack: "MulSetVideoAck" = betterproto.message_field(
        4, group="SubMul"
    )
    set_wiper: "MulSetWiper" = betterproto.message_field(
        5, group="SubMul"
    )
    set_wiper_ack: "MulSetWiperAck" = betterproto.message_field(
        6, group="SubMul"
    )



@dataclass(eq=False, repr=False)
class PerceptionObstaclesT(betterproto.Message):
    label: int = betterproto.int32_field(1)
    num: int = betterproto.int32_field(2)
    points_x: list[int] = betterproto.int32_field(3)
    points_y: list[int] = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class PerceptionObstaclesVisualizationT(betterproto.Message):
    is_heart_beat: int = betterproto.int32_field(1)
    num: int = betterproto.int32_field(2)
    obstacles: list["PerceptionObstaclesT"] = betterproto.message_field(3)
    timestamp: float = betterproto.double_field(4)
    scale: float = betterproto.float_field(5)


@dataclass(eq=False, repr=False)
class PerceptionUniversalBuffT(betterproto.Message):
    perception_type: int = betterproto.int32_field(1)
    perception_len: int = betterproto.int32_field(2)
    universal_buff: list[int] = betterproto.int64_field(3)


@dataclass(eq=False, repr=False)
class MctlPept(betterproto.Message):
    perception_obstacles_visualization: "PerceptionObstaclesVisualizationT" = betterproto.message_field(1, group="SubPeptMsg")
    perception_universal_buff: "PerceptionUniversalBuffT" = (
        betterproto.message_field(2, group="SubPeptMsg")
    )



@dataclass(eq=False, repr=False)
class MsgNull(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class LubaMsg(betterproto.Message):
    msgtype: "MsgCmdType" = betterproto.enum_field(1)
    sender: "MsgDevice" = betterproto.enum_field(2)
    rcver: "MsgDevice" = betterproto.enum_field(3)
    msgattr: "MsgAttr" = betterproto.enum_field(4)
    seqs: int = betterproto.int32_field(5)
    version: int = betterproto.int32_field(6)
    subtype: int = betterproto.int32_field(7)
    net: "DevNet" = betterproto.message_field(
        8, group="LubaSubMsg"
    )
    sys: "MctlSys" = betterproto.message_field(
        10, group="LubaSubMsg"
    )
    nav: "MctlNav" = betterproto.message_field(
        11, group="LubaSubMsg"
    )
    driver: "MctlDriver" = betterproto.message_field(
        12, group="LubaSubMsg"
    )
    ota: "MctlOta" = betterproto.message_field(
        13, group="LubaSubMsg"
    )
    mul: "SocMul" = betterproto.message_field(
        14, group="LubaSubMsg"
    )
    null: "MsgNull" = betterproto.message_field(
        16, group="LubaSubMsg"
    )
    pept: "MctlPept" = betterproto.message_field(
        17, group="LubaSubMsg"
    )
    base: "BaseStation" = betterproto.message_field(
        18, group="LubaSubMsg"
    )
    timestamp: int = betterproto.uint64_field(15)



import asyncio
from collections.abc import Awaitable, Callable
import logging
from typing import Any, cast
from uuid import UUID

import betterproto
from bleak import BleakGATTCharacteristic, BleakGATTServiceCollection, BLEDevice
from bleak.exc import BleakDBusError
from bleak_retry_connector import (
    BLEAK_RETRY_EXCEPTIONS,
    BleakClientWithServiceCache,
    BleakNotFoundError,
    establish_connection,
)

from pymammotion.bluetooth import BleMessage
from pymammotion.data.state_manager import StateManager
from pymammotion.mammotion.commands.mammotion_command import MammotionCommand
from pymammotion.mammotion.devices.base import MammotionBaseDevice
from pymammotion.proto import LubaMsg, has_field

DBUS_ERROR_BACKOFF_TIME = 0.25

DISCONNECT_DELAY = 10


_LOGGER = logging.getLogger(__name__)


class CharacteristicMissingError(Exception):
    """Raised when a characteristic is missing."""


def _sb_uuid(comms_type: str = "service") -> UUID | str:
    """Return Mammotion UUID.

    Args:
        comms_type (str): The type of communication (tx, rx, or service).

    Returns:
        UUID | str: The UUID for the specified communication type or an error message.

    """
    _uuid = {"tx": "ff01", "rx": "ff02", "service": "2A05"}

    if comms_type in _uuid:
        return UUID(f"0000{_uuid[comms_type]}-0000-1000-8000-00805f9b34fb")

    return "Incorrect type, choose between: tx, rx or service"


READ_CHAR_UUID = _sb_uuid(comms_type="rx")
WRITE_CHAR_UUID = _sb_uuid(comms_type="tx")


def _handle_timeout(fut: asyncio.Future[None]) -> None:
    """Handle a timeout."""
    if not fut.done():
        fut.set_exception(asyncio.TimeoutError)


async def _handle_retry(fut: asyncio.Future[None], func, command: bytes) -> None:
    """Handle a retry."""
    if not fut.done():
        await func(command)


class MammotionBaseBLEDevice(MammotionBaseDevice):
    """Base class for Mammotion BLE devices."""

    def __init__(self, state_manager: StateManager, device: BLEDevice, interface: int = 0, **kwargs: Any) -> None:
        """Initialize MammotionBaseBLEDevice."""
        super().__init__(state_manager)
        self._disconnect_strategy = True
        self._ble_sync_task = None
        self._prev_notification = None
        self._interface = f"hci{interface}"
        self.ble_device = device
        self._client: BleakClientWithServiceCache | None = None
        self._read_char: BleakGATTCharacteristic | int | str | UUID = 0
        self._write_char: BleakGATTCharacteristic | int | str | UUID = 0
        self._disconnect_timer: asyncio.TimerHandle | None = None
        self._message: BleMessage | None = None
        self._commands: MammotionCommand = MammotionCommand(device.name, 1)
        self.command_queue = asyncio.Queue()
        self._expected_disconnect = False
        self._connect_lock = asyncio.Lock()
        self._operation_lock = asyncio.Lock()
        self._key: str | None = None
        self.set_queue_callback(self.queue_command)
        self._state_manager.ble_gethash_ack_callback = self.datahash_response
        self._state_manager.ble_get_commondata_ack_callback = self.commdata_response
        self._state_manager.ble_get_plan_callback = self.plan_callback
        loop = asyncio.get_event_loop()
        loop.create_task(self.process_queue())

    def set_notification_callback(self, func: Callable[[tuple[str, Any | None]], Awaitable[None]]) -> None:
        self._state_manager.ble_on_notification_callback = func

    def set_queue_callback(self, func: Callable[[str, dict[str, Any]], Awaitable[bytes]]) -> None:
        self._state_manager.ble_queue_command_callback = func

    def update_device(self, device: BLEDevice) -> None:
        """Update the BLE device."""
        self.ble_device = device

    async def _ble_sync(self) -> None:
        if self._client is not None and self._client.is_connected:
            command_bytes = self._commands.send_todev_ble_sync(2)
            await self._message.post_custom_data_bytes(command_bytes)

    async def run_periodic_sync_task(self) -> None:
        """Send ble sync to robot."""
        try:
            await self._ble_sync()
        finally:
            self.schedule_ble_sync()

    def schedule_ble_sync(self) -> None:
        """Periodically sync to keep connection alive."""
        if self._client is not None and self._client.is_connected:
            self._ble_sync_task = self.loop.call_later(
                130, lambda: asyncio.ensure_future(self.run_periodic_sync_task())
            )

    async def stop(self) -> None:
        """Stop all tasks and disconnect."""
        if self._ble_sync_task:
            self._ble_sync_task.cancel()
        if self._client is not None:
            await self._client.disconnect()

    async def queue_command(self, key: str, **kwargs: Any) -> bytes | None:
        # Create a future to hold the result
        _LOGGER.debug("Queueing command: %s", key)
        future = asyncio.Future()
        # Put the command in the queue as a tuple (key, command, future)
        command_bytes = getattr(self._commands, key)(**kwargs)
        await self.command_queue.put((key, command_bytes, future))
        # Wait for the future to be resolved
        return await future
        # return await self._send_command_with_args(key, **kwargs)

    async def process_queue(self) -> None:
        while True:
            # Get the next item from the queue
            key, command, future = await self.command_queue.get()
            try:
                # Process the command using _execute_command_locked
                result = await self._send_command_locked(key, command)
                # Set the result on the future
                future.set_result(result)
            except Exception as ex:
                # Set the exception on the future if something goes wrong
                future.set_exception(ex)
            finally:
                # Mark the task as done
                self.command_queue.task_done()

    async def _send_command_with_args(self, key: str, **kwargs) -> bytes | None:
        """Send command to device and read response."""
        if self._operation_lock.locked():
            _LOGGER.debug(
                "%s: Operation already in progress, waiting for it to complete; RSSI: %s",
                self.name,
                self.rssi,
            )
        async with self._operation_lock:
            try:
                command_bytes = getattr(self._commands, key)(**kwargs)
                return await self._send_command_locked(key, command_bytes)
            except BleakNotFoundError:
                _LOGGER.exception(
                    "%s: device not found, no longer in range, or poor RSSI: %s",
                    self.name,
                    self.rssi,
                )
                raise
            except CharacteristicMissingError as ex:
                _LOGGER.debug(
                    "%s: characteristic missing: %s; RSSI: %s",
                    self.name,
                    ex,
                    self.rssi,
                    exc_info=True,
                )
            except BLEAK_RETRY_EXCEPTIONS:
                _LOGGER.debug("%s: communication failed with:", self.name, exc_info=True)
        return

    async def _send_command(self, key: str, retry: int | None = None) -> bytes | None:
        """Send command to device and read response."""
        if self._operation_lock.locked():
            _LOGGER.debug(
                "%s: Operation already in progress, waiting for it to complete; RSSI: %s",
                self.name,
                self.rssi,
            )
        async with self._operation_lock:
            try:
                command_bytes = getattr(self._commands, key)()
                return await self._send_command_locked(key, command_bytes)
            except BleakNotFoundError:
                _LOGGER.exception(
                    "%s: device not found, no longer in range, or poor RSSI: %s",
                    self.name,
                    self.rssi,
                )
                raise
            except CharacteristicMissingError as ex:
                _LOGGER.debug(
                    "%s: characteristic missing: %s; RSSI: %s",
                    self.name,
                    ex,
                    self.rssi,
                    exc_info=True,
                )
            except BLEAK_RETRY_EXCEPTIONS:
                _LOGGER.debug("%s: communication failed with:", self.name, exc_info=True)
        return

    @property
    def name(self) -> str:
        """Return device name."""
        return f"{self.ble_device.name} ({self.ble_device.address})"

    @property
    def rssi(self) -> int:
        """Return RSSI of device."""
        try:
            return cast(self.mower.sys.toapp_report_data.connect.ble_rssi, int)
        finally:
            return 0

    async def _ensure_connected(self) -> None:
        """Ensure connection to device is established."""
        if self._connect_lock.locked():
            _LOGGER.debug(
                "%s: Connection already in progress, waiting for it to complete; RSSI: %s",
                self.name,
                self.rssi,
            )
        if self._client and self._client.is_connected:
            _LOGGER.debug(
                "%s: Already connected before obtaining lock, resetting timer; RSSI: %s",
                self.name,
                self.rssi,
            )
            self._reset_disconnect_timer()
            return
        async with self._connect_lock:
            # Check again while holding the lock
            if self._client and self._client.is_connected:
                _LOGGER.debug(
                    "%s: Already connected after obtaining lock, resetting timer; RSSI: %s",
                    self.name,
                    self.rssi,
                )
                self._reset_disconnect_timer()
                return
            _LOGGER.debug("%s: Connecting; RSSI: %s", self.name, self.rssi)
            client: BleakClientWithServiceCache = await establish_connection(
                BleakClientWithServiceCache,
                self.ble_device,
                self.name,
                self._disconnected,
                max_attempts=10,
                ble_device_callback=lambda: self.ble_device,
            )
            _LOGGER.debug("%s: Connected; RSSI: %s", self.name, self.rssi)
            self._client = client
            self._message = BleMessage(client)

            try:
                self._resolve_characteristics(client.services)
            except CharacteristicMissingError as ex:
                _LOGGER.debug(
                    "%s: characteristic missing, clearing cache: %s; RSSI: %s",
                    self.name,
                    ex,
                    self.rssi,
                    exc_info=True,
                )
                await client.clear_cache()
                self._cancel_disconnect_timer()
                await self._execute_disconnect_with_lock()
                raise

            _LOGGER.debug(
                "%s: Starting notify and disconnect timer; RSSI: %s",
                self.name,
                self.rssi,
            )
            self._reset_disconnect_timer()
            await self._start_notify()
            await self._ble_sync()
            self.schedule_ble_sync()

    async def _send_command_locked(self, key: str, command: bytes) -> bytes:
        """Send command to device and read response."""
        await self._ensure_connected()
        try:
            return await self._execute_command_locked(key, command)
        except BleakDBusError as ex:
            # Disconnect so we can reset state and try again
            await asyncio.sleep(DBUS_ERROR_BACKOFF_TIME)
            _LOGGER.debug(
                "%s: RSSI: %s; Backing off %ss; Disconnecting due to error: %s",
                self.name,
                self.rssi,
                DBUS_ERROR_BACKOFF_TIME,
                ex,
            )
            await self._execute_forced_disconnect()
            raise
        except BLEAK_RETRY_EXCEPTIONS as ex:
            # Disconnect so we can reset state and try again
            _LOGGER.debug("%s: RSSI: %s; Disconnecting due to error: %s", self.name, self.rssi, ex)
            await self._execute_forced_disconnect()
            raise

    async def _notification_handler(self, _sender: BleakGATTCharacteristic, data: bytearray) -> None:
        """Handle notification responses."""
        if self._message is None:
            return
        result = self._message.parseNotification(data)
        if result == 0:
            data = await self._message.parseBlufiNotifyData(True)
            try:
                self._update_raw_data(data)
            except (KeyError, ValueError, IndexError, UnicodeDecodeError):
                _LOGGER.exception("Error parsing message %s", data)
                data = b""
            finally:
                self._message.clear_notification()

            _LOGGER.debug("%s: Received notification: %s", self.name, data)
        else:
            return
        new_msg = LubaMsg().parse(data)
        if betterproto.serialized_on_wire(new_msg.net):
            if new_msg.net.todev_ble_sync != 0 or has_field(new_msg.net.toapp_wifi_iot_status):
                if has_field(new_msg.net.toapp_wifi_iot_status) and self._commands.get_device_product_key() == "":
                    self._commands.set_device_product_key(new_msg.net.toapp_wifi_iot_status.productkey)

                return

        await self._state_manager.notification(new_msg)
        # may or may not be correct, some work could be done here to correctly match responses
        if self._notify_future and not self._notify_future.done():
            self._notify_future.set_result(data)

        if self._execute_timed_disconnect is None:
            await self._execute_forced_disconnect()

        self._reset_disconnect_timer()

    async def _start_notify(self) -> None:
        """Start notification."""
        _LOGGER.debug("%s: Subscribe to notifications; RSSI: %s", self.name, self.rssi)
        await self._client.start_notify(self._read_char, self._notification_handler)

    async def _execute_command_locked(self, key: str, command: bytes) -> bytes:
        """Execute command and read response."""
        assert self._client is not None
        self._notify_future = self.loop.create_future()
        self._key = key
        _LOGGER.debug("%s: Sending command: %s", self.name, key)
        await self._message.post_custom_data_bytes(command)

        timeout = 1
        timeout_handle = self.loop.call_at(self.loop.time() + timeout, _handle_timeout, self._notify_future)
        timeout_expired = False
        try:
            notify_msg = await self._notify_future
        except asyncio.TimeoutError:
            timeout_expired = True
            notify_msg = b""
        finally:
            if not timeout_expired:
                timeout_handle.cancel()
            self._notify_future = None

        _LOGGER.debug("%s: Notification received: %s", self.name, notify_msg.hex())
        return notify_msg

    def get_address(self) -> str:
        """Return address of device."""
        return self.ble_device.address

    def _resolve_characteristics(self, services: BleakGATTServiceCollection) -> None:
        """Resolve characteristics."""
        self._read_char = services.get_characteristic(READ_CHAR_UUID)
        if not self._read_char:
            _LOGGER.error(CharacteristicMissingError(READ_CHAR_UUID))
        self._write_char = services.get_characteristic(WRITE_CHAR_UUID)
        if not self._write_char:
            _LOGGER.error(CharacteristicMissingError(WRITE_CHAR_UUID))

    def _reset_disconnect_timer(self) -> None:
        """Reset disconnect timer."""
        self._cancel_disconnect_timer()
        self._expected_disconnect = False
        self._disconnect_timer = self.loop.call_later(DISCONNECT_DELAY, self._disconnect_from_timer)

    def _disconnected(self, client: BleakClientWithServiceCache) -> None:
        """Disconnected callback."""
        if self._expected_disconnect:
            _LOGGER.debug("%s: Disconnected from device; RSSI: %s", self.name, self.rssi)
            return
        _LOGGER.warning(
            "%s: Device unexpectedly disconnected; RSSI: %s",
            self.name,
            self.rssi,
        )
        self._cancel_disconnect_timer()
        self._client = None

    def _disconnect_from_timer(self) -> None:
        """Disconnect from device."""
        if self._operation_lock.locked() and self._client.is_connected:
            _LOGGER.debug(
                "%s: Operation in progress, resetting disconnect timer; RSSI: %s",
                self.name,
                self.rssi,
            )
            self._reset_disconnect_timer()
            return
        self._cancel_disconnect_timer()
        self._timed_disconnect_task = asyncio.create_task(self._execute_timed_disconnect())

    def _cancel_disconnect_timer(self) -> None:
        """Cancel disconnect timer."""
        if self._disconnect_timer:
            self._disconnect_timer.cancel()
            self._disconnect_timer = None

    async def _execute_forced_disconnect(self) -> None:
        """Execute forced disconnection."""
        self._cancel_disconnect_timer()
        _LOGGER.debug(
            "%s: Executing forced disconnect",
            self.name,
        )
        await self._execute_disconnect()

    async def _execute_timed_disconnect(self) -> None:
        """Execute timed disconnection."""
        if not self._disconnect_strategy:
            return
        _LOGGER.debug(
            "%s: Executing timed disconnect after timeout of %s",
            self.name,
            DISCONNECT_DELAY,
        )
        await self._execute_disconnect()

    async def _execute_disconnect(self) -> None:
        """Execute disconnection."""
        _LOGGER.debug("%s: Executing disconnect", self.name)
        async with self._connect_lock:
            await self._execute_disconnect_with_lock()

    async def _execute_disconnect_with_lock(self) -> None:
        """Execute disconnection while holding the lock."""
        assert self._connect_lock.locked(), "Lock not held"
        _LOGGER.debug("%s: Executing disconnect with lock", self.name)
        if self._disconnect_timer:  # If the timer was reset, don't disconnect
            _LOGGER.debug("%s: Skipping disconnect as timer reset", self.name)
            return
        client = self._client
        self._expected_disconnect = True

        if not client:
            _LOGGER.debug("%s: Already disconnected", self.name)
            return
        _LOGGER.debug("%s: Disconnecting", self.name)
        try:
            """We reset what command the robot last heard before disconnecting."""
            if client is not None and client.is_connected:
                command_bytes = self._commands.send_todev_ble_sync(2)
                await self._message.post_custom_data_bytes(command_bytes)
                await client.stop_notify(self._read_char)
                await client.disconnect()
        except BLEAK_RETRY_EXCEPTIONS as ex:
            _LOGGER.warning(
                "%s: Error disconnecting: %s; RSSI: %s",
                self.name,
                ex,
                self.rssi,
            )
        else:
            _LOGGER.debug("%s: Disconnect completed successfully", self.name)
        self._client = None

    def set_disconnect_strategy(self, disconnect: bool) -> None:
        self._disconnect_strategy = disconnect



import asyncio
import logging
import os
import socket
import ssl
import time
from typing import Any
from urllib.parse import urlencode, urlparse

import aiohttp
import certifi
from requests import PreparedRequest, adapters, status_codes
from Tea.exceptions import RequiredArgumentException, RetryError
from Tea.model import TeaModel
from Tea.request import TeaRequest
from Tea.response import TeaResponse
from Tea.stream import BaseStream

DEFAULT_CONNECT_TIMEOUT = 5000
DEFAULT_READ_TIMEOUT = 10000
DEFAULT_POOL_SIZE = 10

logger = logging.getLogger("alibabacloud-tea")
logger.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
logger.addHandler(ch)


class TeaCore:
    http_adapter = adapters.HTTPAdapter(pool_connections=DEFAULT_POOL_SIZE, pool_maxsize=DEFAULT_POOL_SIZE * 4)
    https_adapter = adapters.HTTPAdapter(pool_connections=DEFAULT_POOL_SIZE, pool_maxsize=DEFAULT_POOL_SIZE * 4)

    @staticmethod
    def get_adapter(prefix):
        if prefix.upper() == "HTTP":
            return TeaCore.http_adapter
        else:
            return TeaCore.https_adapter

    @staticmethod
    def _prepare_http_debug(request, symbol):
        base = ""
        for key, value in request.headers.items():
            base += f"\n{symbol} {key} : {value}"
        return base

    @staticmethod
    def _do_http_debug(request, response) -> None:
        # logger the request
        url = urlparse(request.url)
        request_base = f"\n> {request.method.upper()} {url.path + url.query} HTTP/1.1"
        logger.debug(request_base + TeaCore._prepare_http_debug(request, ">"))

        # logger the response
        response_base = (
            f"\n< HTTP/1.1 {response.status_code}" f" {status_codes._codes.get(response.status_code)[0].upper()}"
        )
        logger.debug(response_base + TeaCore._prepare_http_debug(response, "<"))

    @staticmethod
    def compose_url(request):
        host = request.headers.get("host")
        if not host:
            raise RequiredArgumentException("endpoint")
        else:
            host = host.rstrip("/")
        protocol = f"{request.protocol.lower()}://"
        pathname = request.pathname

        if host.startswith(("http://", "https://")):
            protocol = ""

        if request.port == 80:
            port = ""
        else:
            port = f":{request.port}"

        url = protocol + host + port + pathname

        if request.query:
            if "?" in url:
                if not url.endswith("&"):
                    url += "&"
            else:
                url += "?"

            encode_query = {}
            for key in request.query:
                value = request.query[key]
                if value is not None:
                    encode_query[key] = str(value)
            url += urlencode(encode_query)
        return url.rstrip("?&")

    @staticmethod
    async def async_do_action(request: TeaRequest, runtime_option=None) -> TeaResponse:
        runtime_option = runtime_option or {}

        url = TeaCore.compose_url(request)
        verify = not runtime_option.get("ignoreSSL", False)

        timeout = runtime_option.get("timeout")
        connect_timeout = runtime_option.get("connectTimeout") or timeout or DEFAULT_CONNECT_TIMEOUT
        read_timeout = runtime_option.get("readTimeout") or timeout or DEFAULT_READ_TIMEOUT

        connect_timeout, read_timeout = (int(connect_timeout) / 1000, int(read_timeout) / 1000)

        proxy = None
        if request.protocol.upper() == "HTTP":
            proxy = runtime_option.get("httpProxy")
            if not proxy:
                proxy = os.environ.get("HTTP_PROXY") or os.environ.get("http_proxy")
        elif request.protocol.upper() == "HTTPS":
            proxy = runtime_option.get("httpsProxy")
            if not proxy:
                proxy = os.environ.get("HTTPS_PROXY") or os.environ.get("https_proxy")

        connector = None
        ca_cert = certifi.where()
        if ca_cert and request.protocol.upper() == "HTTPS":
            loop = asyncio.get_event_loop()

            ssl_context = await loop.run_in_executor(None, ssl.create_default_context, ssl.Purpose.SERVER_AUTH)
            await loop.run_in_executor(None, ssl_context.load_verify_locations, ca_cert)
            connector = aiohttp.TCPConnector(
                ssl=ssl_context,
                family=socket.AF_INET,
            )
        else:
            verify = False

        timeout = aiohttp.ClientTimeout(sock_read=read_timeout, sock_connect=connect_timeout)
        async with aiohttp.ClientSession(connector=connector) as s:
            body = b""
            if isinstance(request.body, BaseStream):
                for content in request.body:
                    body += content
            elif isinstance(request.body, str):
                body = request.body.encode("utf-8")
            else:
                body = request.body
            try:
                async with s.request(
                    request.method, url, data=body, headers=request.headers, ssl=verify, proxy=proxy, timeout=timeout
                ) as response:
                    tea_resp = TeaResponse()
                    tea_resp.body = await response.read()
                    tea_resp.headers = {k.lower(): v for k, v in response.headers.items()}
                    tea_resp.status_code = response.status
                    tea_resp.status_message = response.reason
                    tea_resp.response = response
            except OSError as e:
                raise RetryError(str(e))
        return tea_resp

    @staticmethod
    def do_action(request: TeaRequest, runtime_option=None) -> TeaResponse:
        url = TeaCore.compose_url(request)

        runtime_option = runtime_option or {}

        verify = not runtime_option.get("ignoreSSL", False)
        if verify:
            verify = runtime_option.get("ca", True) if runtime_option.get("ca", True) is not None else True
        cert = runtime_option.get("cert", None)

        timeout = runtime_option.get("timeout")
        connect_timeout = runtime_option.get("connectTimeout") or timeout or DEFAULT_CONNECT_TIMEOUT
        read_timeout = runtime_option.get("readTimeout") or timeout or DEFAULT_READ_TIMEOUT

        timeout = (int(connect_timeout) / 1000, int(read_timeout) / 1000)

        if isinstance(request.body, str):
            request.body = request.body.encode("utf-8")

        p = PreparedRequest()
        p.prepare(
            method=request.method.upper(),
            url=url,
            data=request.body,
            headers=request.headers,
        )

        proxies = {}
        http_proxy = runtime_option.get("httpProxy")
        https_proxy = runtime_option.get("httpsProxy")
        no_proxy = runtime_option.get("noProxy")

        if not http_proxy:
            http_proxy = os.environ.get("HTTP_PROXY") or os.environ.get("http_proxy")
        if not https_proxy:
            https_proxy = os.environ.get("HTTPS_PROXY") or os.environ.get("https_proxy")

        if http_proxy:
            proxies["http"] = http_proxy
        if https_proxy:
            proxies["https"] = https_proxy
        if no_proxy:
            proxies["no_proxy"] = no_proxy

        adapter = TeaCore.get_adapter(request.protocol)
        try:
            resp = adapter.send(
                p,
                proxies=proxies,
                timeout=timeout,
                verify=verify,
                cert=cert,
            )
        except OSError as e:
            raise RetryError(str(e))

        debug = runtime_option.get("debug") or os.getenv("DEBUG")
        if debug and debug.lower() == "sdk":
            TeaCore._do_http_debug(p, resp)

        response = TeaResponse()
        response.status_message = resp.reason
        response.status_code = resp.status_code
        response.headers = {k.lower(): v for k, v in resp.headers.items()}
        response.body = resp.content
        response.response = resp
        return response

    @staticmethod
    def get_response_body(resp) -> str:
        return resp.content.decode("utf-8")

    @staticmethod
    def allow_retry(dic, retry_times, now=None) -> bool:
        if retry_times == 0:
            return True
        if dic is None or not dic.__contains__("maxAttempts") or dic.get("retryable") is not True and retry_times >= 1:
            return False
        else:
            retry = 0 if dic.get("maxAttempts") is None else int(dic.get("maxAttempts"))
        return retry >= retry_times

    @staticmethod
    def get_backoff_time(dic, retry_times) -> int:
        default_back_off_time = 0
        if dic is None or not dic.get("policy") or dic.get("policy") == "no":
            return default_back_off_time

        back_off_time = dic.get("period", default_back_off_time)
        if not isinstance(back_off_time, int) and not (isinstance(back_off_time, str) and back_off_time.isdigit()):
            return default_back_off_time

        back_off_time = int(back_off_time)
        if back_off_time < 0:
            return retry_times

        return back_off_time

    @staticmethod
    async def sleep_async(t) -> None:
        await asyncio.sleep(t)

    @staticmethod
    def sleep(t) -> None:
        time.sleep(t)

    @staticmethod
    def is_retryable(ex) -> bool:
        return isinstance(ex, RetryError)

    @staticmethod
    def bytes_readable(body):
        return body

    @staticmethod
    def merge(*dic_list) -> dict:
        dic_result = {}
        for item in dic_list:
            if isinstance(item, dict):
                dic_result.update(item)
            elif isinstance(item, TeaModel):
                dic_result.update(item.to_map())
        return dic_result

    @staticmethod
    def to_map(model: TeaModel | None) -> dict[str, Any]:
        if isinstance(model, TeaModel):
            return model.to_map()
        else:
            return dict()

    @staticmethod
    def from_map(model: TeaModel, dic: dict[str, Any]) -> TeaModel:
        if isinstance(model, TeaModel):
            try:
                return model.from_map(dic)
            except Exception:
                model._map = dic
                return model
        else:
            return model



from dataclasses import dataclass, field
from enum import IntEnum

from mashumaro.mixins.orjson import DataClassORJSONMixin

from pymammotion.proto import NavGetCommDataAck, NavGetHashListAck, SvgMessageAckT


class PathType(IntEnum):
    """Path types for common data."""

    AREA = 0
    OBSTACLE = 1
    PATH = 2
    LINE = 10
    DUMP = 12
    SVG = 13


@dataclass
class CommDataCouple:
    x: float = 0.0
    y: float = 0.0


@dataclass
class AreaLabelName(DataClassORJSONMixin):
    label: str = ""


@dataclass
class NavGetCommData(DataClassORJSONMixin):
    pver: int = 0
    sub_cmd: int = 0
    result: int = 0
    action: int = 0
    type: int = 0
    hash: int = 0
    paternal_hash_a: int = 0
    paternal_hash_b: int = 0
    total_frame: int = 0
    current_frame: int = 0
    data_hash: int = 0
    data_len: int = 0
    data_couple: list["CommDataCouple"] = field(default_factory=list)
    reserved: str = ""
    area_label: "AreaLabelName" = field(default_factory=AreaLabelName)


@dataclass
class SvgMessageData(DataClassORJSONMixin):
    x_move: float = 0.0
    y_move: float = 0.0
    scale: float = 0.0
    rotate: float = 0.0
    base_width_m: float = 0.0
    base_width_pix: int = 0
    base_height_m: float = 0.0
    base_height_pix: int = 0
    data_count: int = 0
    hide_svg: bool = False
    name_count: int = 0
    svg_file_name: str = ""
    svg_file_data: str = ""


@dataclass
class SvgMessage(DataClassORJSONMixin):
    pver: int = 0
    sub_cmd: int = 0
    total_frame: int = 0
    current_frame: int = 0
    data_hash: int = 0
    paternal_hash_a: int = 0
    type: int = 0
    result: int = 0
    svg_message: "SvgMessageData" = field(default_factory=SvgMessageData)


@dataclass
class FrameList(DataClassORJSONMixin):
    total_frame: int = 0
    sub_cmd: int = 0
    data: list[NavGetCommData | SvgMessage] = field(default_factory=list)


@dataclass
class Plan(DataClassORJSONMixin):
    pver: int = 0
    sub_cmd: int = 2
    area: int = 0
    work_time: int = 0
    version: str = ""
    id: str = ""
    user_id: str = ""
    device_id: str = ""
    plan_id: str = ""
    task_id: str = ""
    job_id: str = ""
    start_time: str = ""
    end_time: str = ""
    week: int = 0
    knife_height: int = 0
    model: int = 0
    edge_mode: int = 0
    required_time: int = 0
    route_angle: int = 0
    route_model: int = 0
    route_spacing: int = 0
    ultrasonic_barrier: int = 0
    total_plan_num: int = 0
    plan_index: int = 0
    result: int = 0
    speed: float = 0.0
    task_name: str = ""
    job_name: str = ""
    zone_hashs: list[int] = field(default_factory=list)
    reserved: str = ""
    start_date: str = ""
    end_date: str = ""
    trigger_type: int = 0
    day: int = 0
    weeks: list[int] = field(default_factory=list)
    remained_seconds: int = 0
    toward_mode: int = 0
    toward_included_angle: int = 0


@dataclass(eq=False, repr=False)
class NavGetHashListData(DataClassORJSONMixin):
    """Dataclass for NavGetHashListData."""

    pver: int = 0
    sub_cmd: int = 0
    total_frame: int = 0
    current_frame: int = 0
    data_hash: int = 0
    hash_len: int = 0
    reserved: str = ""
    result: int = 0
    data_couple: list[int] = field(default_factory=list)


@dataclass
class RootHashList(DataClassORJSONMixin):
    total_frame: int = 0
    sub_cmd: int = 0
    data: list[NavGetHashListData] = field(default_factory=list)


@dataclass
class AreaHashNameList(DataClassORJSONMixin):
    """Wrapper so we can serialize to and from dict."""

    name: str
    hash: int


@dataclass
class HashList(DataClassORJSONMixin):
    """stores our map data.
    [hashID, FrameList].
    hashlist for all our hashIDs for verification
    """

    root_hash_lists: list[RootHashList] = field(default_factory=list)
    area: dict[int, FrameList] = field(default_factory=dict)  # type 0
    path: dict[int, FrameList] = field(default_factory=dict)  # type 2
    obstacle: dict[int, FrameList] = field(default_factory=dict)  # type 1
    dump: dict[int, FrameList] = field(default_factory=dict)  # type 12? / sub cmd 4
    svg: dict[int, FrameList] = field(default_factory=dict)  # type 13
    line: dict[int, FrameList] = field(default_factory=dict)  # type 10 possibly breakpoint? / sub cmd 3
    plan: dict[int, Plan] = field(default_factory=dict)
    area_name: list[AreaHashNameList] = field(default_factory=list)

    def update_hash_lists(self, hashlist: list[int]) -> None:
        self.area = {hash_id: frames for hash_id, frames in self.area.items() if hash_id in hashlist}
        self.path = {hash_id: frames for hash_id, frames in self.path.items() if hash_id in hashlist}
        self.obstacle = {hash_id: frames for hash_id, frames in self.obstacle.items() if hash_id in hashlist}
        self.dump = {hash_id: frames for hash_id, frames in self.dump.items() if hash_id in hashlist}
        self.svg = {hash_id: frames for hash_id, frames in self.svg.items() if hash_id in hashlist}
        self.area_name = [
            area_item
            for area_item in self.area_name
            if area_item.hash in self.area.keys() or area_item.hash in hashlist
        ]

    @property
    def hashlist(self) -> list[int]:
        if not self.root_hash_lists:
            return []
        # Combine data_couple from all RootHashLists
        return [i for root_list in self.root_hash_lists for obj in root_list.data for i in obj.data_couple]

    def missing_hashlist(self, sub_cmd: int = 0) -> list[int]:
        """Return missing hashlist."""
        all_hash_ids = set(self.area.keys()).union(
            self.path.keys(), self.obstacle.keys(), self.dump.keys(), self.svg.keys()
        )
        if sub_cmd == 3:
            all_hash_ids = set(self.line.keys())
        return [
            i
            for root_list in self.root_hash_lists
            for obj in root_list.data
            if root_list.sub_cmd == sub_cmd
            for i in obj.data_couple
            if i not in all_hash_ids
        ]

    def missing_root_hash_frame(self, hash_list: NavGetHashListAck) -> list[int]:
        """Return missing root hash frame."""
        target_root_list = next(
            (
                rhl
                for rhl in self.root_hash_lists
                if rhl.total_frame == hash_list.total_frame and rhl.sub_cmd == hash_list.sub_cmd
            ),
            None,
        )
        if target_root_list is None:
            return []

        return self.find_missing_frames(target_root_list)

    def update_root_hash_list(self, hash_list: NavGetHashListData) -> None:
        target_root_list = next(
            (
                rhl
                for rhl in self.root_hash_lists
                if rhl.total_frame == hash_list.total_frame and rhl.sub_cmd == hash_list.sub_cmd
            ),
            None,
        )

        if target_root_list is None:
            # Create new RootHashList if none exists for this total_frame
            new_root_list = RootHashList(total_frame=hash_list.total_frame, sub_cmd=hash_list.sub_cmd, data=[hash_list])
            self.root_hash_lists.append(new_root_list)
            return

        for index, obj in enumerate(target_root_list.data):
            if obj.current_frame == hash_list.current_frame:
                # Replace the item if current_frame matches
                target_root_list.data[index] = hash_list
                return

        # If no match was found, append the new item
        target_root_list.data.append(hash_list)

    def missing_hash_frame(self, hash_ack: NavGetHashListAck) -> list[int]:
        """Returns a combined list of all missing frames across all RootHashLists."""
        missing_frames = []
        filtered_lists = [rl for rl in self.root_hash_lists if rl.sub_cmd == hash_ack.sub_cmd]
        for root_list in filtered_lists:
            missing = self.find_missing_frames(root_list)
            if missing:
                missing_frames.extend(missing)
        return missing_frames

    def missing_frame(self, hash_data: NavGetCommDataAck | SvgMessageAckT) -> list[int]:
        if hash_data.type == PathType.AREA:
            return self.find_missing_frames(self.area.get(hash_data.hash))

        if hash_data.type == PathType.OBSTACLE:
            return self.find_missing_frames(self.obstacle.get(hash_data.hash))

        if hash_data.type == PathType.PATH:
            return self.find_missing_frames(self.path.get(hash_data.hash))

        if hash_data.type == PathType.LINE:
            return self.find_missing_frames(self.line.get(hash_data.hash))

        if hash_data.type == PathType.DUMP:
            return self.find_missing_frames(self.dump.get(hash_data.hash))

        if hash_data.type == PathType.SVG:
            return self.find_missing_frames(self.svg.get(hash_data.data_hash))

        return []

    def update_plan(self, plan: Plan) -> None:
        self.plan[plan.plan_index] = plan

    def update(self, hash_data: NavGetCommData | SvgMessage) -> bool:
        """Update the map data."""

        if hash_data.type == PathType.AREA:
            existing_name = next((area for area in self.area_name if area.hash == hash_data.hash), None)
            if not existing_name:
                name = f"area {len(self.area_name)+1}" if hash_data.area_label is None else hash_data.area_label.label
                self.area_name.append(AreaHashNameList(name=name, hash=hash_data.hash))
            result = self._add_hash_data(self.area, hash_data)
            self.update_hash_lists(self.hashlist)
            return result

        if hash_data.type == PathType.OBSTACLE:
            return self._add_hash_data(self.obstacle, hash_data)

        if hash_data.type == PathType.PATH:
            return self._add_hash_data(self.path, hash_data)

        if hash_data.type == PathType.LINE:
            return self._add_hash_data(self.line, hash_data)

        if hash_data.type == PathType.DUMP:
            return self._add_hash_data(self.dump, hash_data)

        if hash_data.type == PathType.SVG:
            return self._add_hash_data(self.svg, hash_data)

        return False

    @staticmethod
    def find_missing_frames(frame_list: FrameList | RootHashList) -> list[int]:
        if frame_list is None:
            return []

        if frame_list.total_frame == len(frame_list.data):
            return []
        number_list = list(range(1, frame_list.total_frame + 1))

        current_frames = {frame.current_frame for frame in frame_list.data}
        missing_numbers = [num for num in number_list if num not in current_frames]
        return missing_numbers

    @staticmethod
    def _add_hash_data(hash_dict: dict[int, FrameList], hash_data: NavGetCommData | SvgMessage) -> bool:
        if isinstance(hash_data, SvgMessage):
            if hash_dict.get(hash_data.data_hash) is None:
                hash_dict[hash_data.data_hash] = FrameList(total_frame=hash_data.total_frame, data=[hash_data])
                return True

            if hash_data not in hash_dict[hash_data.data_hash].data:
                exists = next(
                    (
                        rhl
                        for rhl in hash_dict[hash_data.data_hash].data
                        if rhl.current_frame == hash_data.current_frame
                    ),
                    None,
                )
                if exists:
                    return True
                hash_dict[hash_data.data_hash].data.append(hash_data)
                return True
            return False

        if hash_dict.get(hash_data.hash) is None:
            hash_dict[hash_data.hash] = FrameList(total_frame=hash_data.total_frame, data=[hash_data])
            return True

        if hash_data not in hash_dict[hash_data.hash].data:
            exists = next(
                (rhl for rhl in hash_dict[hash_data.hash].data if rhl.current_frame == hash_data.current_frame),
                None,
            )
            if exists:
                return True
            hash_dict[hash_data.hash].data.append(hash_data)
            return True
        return False



from dataclasses import dataclass
from enum import Enum
from typing import Literal

from mashumaro.mixins.orjson import DataClassORJSONMixin


@dataclass
class GroupIdListItem(DataClassORJSONMixin):
    groupId: str
    groupType: Literal["ISOLATION", "ILOP_APP"]


# are there other values?
class StatusType(Enum):
    CONNECTED = 1
    DISCONNECTED = 3


@dataclass
class Status(DataClassORJSONMixin):
    time: int
    value: StatusType


@dataclass
class Params(DataClassORJSONMixin):
    groupIdList: list[GroupIdListItem]
    netType: Literal["NET_WIFI", "NET_MNET"]
    activeTime: int
    ip: str
    aliyunCommodityCode: Literal["iothub_senior"]
    categoryKey: Literal["LawnMower"]
    nodeType: Literal["DEVICE"]
    productKey: str
    statusLast: int
    deviceName: str
    iotId: str
    namespace: str
    tenantId: str
    thingType: Literal["DEVICE"]
    tenantInstanceId: str
    categoryId: int
    status: Status


@dataclass
class ThingStatusMessage(DataClassORJSONMixin):
    method: Literal["thing.status"]
    id: str
    params: Params
    version: Literal["1.0"]



from dataclasses import dataclass

from mashumaro.mixins.orjson import DataClassORJSONMixin


@dataclass
class SessionOauthToken(DataClassORJSONMixin):
    identityId: str
    refreshTokenExpire: int
    iotToken: str
    iotTokenExpire: int
    refreshToken: str


@dataclass
class SessionByAuthCodeResponse(DataClassORJSONMixin):
    code: int
    data: SessionOauthToken | None = None
    token_issued_at: int | None = None



from pymammotion.mammotion.commands.messages.driver import MessageDriver
from pymammotion.mammotion.commands.messages.media import MessageMedia
from pymammotion.mammotion.commands.messages.navigation import MessageNavigation
from pymammotion.mammotion.commands.messages.network import MessageNetwork
from pymammotion.mammotion.commands.messages.ota import MessageOta
from pymammotion.mammotion.commands.messages.system import MessageSystem
from pymammotion.mammotion.commands.messages.video import MessageVideo
from pymammotion.utility.movement import get_percent, transform_both_speeds


class MammotionCommand(
    MessageSystem, MessageNavigation, MessageNetwork, MessageOta, MessageVideo, MessageMedia, MessageDriver
):
    """MQTT commands for Luba."""

    def __init__(self, device_name: str, user_account: int) -> None:
        self._device_name = device_name
        self._product_key = ""
        self.user_account = user_account

    def get_device_name(self) -> str:
        """Get device name."""
        return self._device_name

    def get_device_product_key(self) -> str:
        return self._product_key

    def set_device_product_key(self, product_key: str) -> None:
        self._product_key = product_key

    def move_forward(self, linear: float) -> bytes:
        """Move forward. values 0.0 1.0."""
        linear_percent = get_percent(abs(linear * 100))
        (linear_speed, angular_speed) = transform_both_speeds(90.0, 0.0, linear_percent, 0.0)
        return self.send_movement(linear_speed=linear_speed, angular_speed=angular_speed)

    def move_back(self, linear: float) -> bytes:
        """Move back. values 0.0 1.0."""
        linear_percent = get_percent(abs(linear * 100))
        (linear_speed, angular_speed) = transform_both_speeds(270.0, 0.0, linear_percent, 0.0)
        return self.send_movement(linear_speed=linear_speed, angular_speed=angular_speed)

    def move_left(self, angular: float) -> bytes:
        """Move forward. values 0.0 1.0."""
        angular_percent = get_percent(abs(angular * 100))
        (linear_speed, angular_speed) = transform_both_speeds(0.0, 180.0, 0.0, angular_percent)
        return self.send_movement(linear_speed=linear_speed, angular_speed=angular_speed)

    def move_right(self, angular: float) -> bytes:
        """Move back. values 0.0 1.0."""
        angular_percent = get_percent(abs(angular * 100))
        (linear_speed, angular_speed) = transform_both_speeds(0.0, 0.0, 0.0, angular_percent)
        return self.send_movement(linear_speed=linear_speed, angular_speed=angular_speed)
